@@ -104,7 +104,7 @@ nautilus_content_loser_destroy (GtkObject *object)
 	g_free (view->details->uri);
 	g_free (view->details);
 	
-	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (object));
+	NAUTILUS_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
 
 /**
