@@ -61,36 +61,36 @@
 #define OAF_ID "OAFIID:trilobite_eazel_install_service:8ff6e815-1992-437c-9771-d932db3b4a17"
 
 /* Popt stuff */
-int     arg_dry_run,
-	arg_http,
-	arg_ftp,
-	arg_local,
-	arg_debug,
-	arg_delay,
-	arg_file,
-	arg_force,
-	arg_upgrade,
-	arg_downgrade,
-	arg_erase,
-	arg_query,
-	arg_revert,
-	arg_ssl_rename,
-	arg_provides,
-	arg_verbose,
-	arg_id,
-	arg_suite,
-	arg_ei2,
-	arg_no_pct,
-	arg_no_auth,
-	arg_silent,
-	arg_machine;
-char    *arg_server,
-	*arg_cgi,
-	*arg_config_file,
-	*arg_package_list,
-	*arg_username,
-	*arg_root,
-	*arg_batch,
+int     arg_dry_run = 0,
+	arg_http = 0,
+	arg_ftp = 0,
+	arg_local = 0,
+	arg_debug = 0,
+	arg_delay = 0,
+	arg_file = 0,
+	arg_force = 0,
+	arg_upgrade = 0,
+	arg_downgrade = 0,
+	arg_erase = 0,
+	arg_query = 0,
+	arg_revert = 0,
+	arg_ssl_rename = 0,
+	arg_provides = 0,
+	arg_verbose = 0,
+	arg_id = 0,
+	arg_suite = 0,
+	arg_ei2 = 0,
+	arg_no_pct = 0,
+	arg_no_auth = 0,
+	arg_silent = 0,
+	arg_machine = 0;
+char    *arg_server = NULL,
+	*arg_cgi = NULL,
+	*arg_config_file = NULL,
+	*arg_package_list = NULL,
+	*arg_username = NULL,
+	*arg_root = NULL,
+	*arg_batch = NULL,
 	*arg_version = NULL;
 
 /* Yeahyeah, but this was initially a test tool,
@@ -112,7 +112,6 @@ static const struct poptOption options[] = {
 	{"erase", 'e', POPT_ARG_NONE, &arg_erase, 0, N_("Erase packages"), NULL},
 	{"ei2", '\0', POPT_ARG_NONE, &arg_ei2, 0, N_("enable ei2"), NULL},
 	{"file",'\0', POPT_ARG_NONE, &arg_file, 0, N_("RPM args are filename"), NULL},
-	{"force", 'F', POPT_ARG_NONE, &arg_force, 0, N_("Force install"), NULL},
 	{"ftp", 'f', POPT_ARG_NONE, &arg_ftp, 0, N_("Use ftp"), NULL},
 	{"local", 'l', POPT_ARG_NONE, &arg_local, 0, N_("Use local"), NULL},
 	{"http", 'h', POPT_ARG_NONE, &arg_http, 0, N_("Use http"), NULL},
@@ -123,7 +122,8 @@ static const struct poptOption options[] = {
 	{"packagefile", '\0', POPT_ARG_STRING, &arg_package_list, 0, N_("Specify package file"), NULL},
 	{"provides", '\0', POPT_ARG_NONE, &arg_provides, 0, N_("RPM args are needed files"), NULL},
 	{"query", 'q', POPT_ARG_NONE, &arg_query, 0, N_("Run Query"), NULL},
-	{"revert", 'r', POPT_ARG_NONE, &arg_revert, 0, N_("Revert"), NULL},
+	/* Disabled for 1.0 */
+	/* {"revert", 'r', POPT_ARG_NONE, &arg_revert, 0, N_("Revert"), NULL}, */
 	{"root", '\0', POPT_ARG_STRING, &arg_root, 0, N_("Set root"), NULL},
 	{"server", '\0', POPT_ARG_STRING, &arg_server, 0, N_("Specify server"), NULL},
 	{"silent", '\0', POPT_ARG_NONE, &arg_silent, 0, N_("Dont print too much, just problems and download"), NULL},
@@ -137,14 +137,6 @@ static const struct poptOption options[] = {
 	{NULL, '\0', 0, NULL, 0}
 };
 
-static void tree_helper (EazelInstallCallback *service,
-			 PackageData *pd,		
-			 gchar *indent,
-			 gchar *indent_type,
-			 int indent_level,
-			 char *title,
-			 GList **seen);
-
 #define check_ev(s)                                                \
 if (ev._major!=CORBA_NO_EXCEPTION) {                               \
 	fprintf (stderr, "*** %s: Caught exception %s",            \
@@ -354,7 +346,7 @@ eazel_download_progress_signal (EazelInstallCallback *service,
 		old_pct = pct = 0;
 	} else if (amount != total ) {
 		if (arg_no_pct==0) {
-			pct = amount / (total / 100);
+			pct = total != 0 ? (amount * 100) / total : 100;
 			if (pct > 5) {
 				if (old_pct != pct && pct%5==0) {
 					end = time (NULL);
@@ -419,7 +411,7 @@ eazel_install_progress_signal (EazelInstallCallback *service,
 						 package_num, num_packages,
 						 total_size_completed, total_size,
 						 amount, total,
-						 amount / (total / 100));
+						 total != 0 ? (amount * 100) / total : 100);
 				} else {
 					fprintf (stdout, "\r");
 					fprintf (stdout, _("Uninstalling %s (%d/%d), (%d/%d)b - (%d/%d) = %d%%"), 
@@ -427,7 +419,7 @@ eazel_install_progress_signal (EazelInstallCallback *service,
 						 package_num, num_packages,
 						 total_size_completed, total_size,
 						 amount, total,
-						 amount / (total / 100));
+						 total != 0 ? (amount * 100) / total : 100);
 
 				}
 			}
@@ -451,7 +443,7 @@ eazel_install_progress_signal (EazelInstallCallback *service,
 						 amount, total, 100);
 				} else {
 					fprintf (stdout, "\r");
-					fprintf (stdout, _("Unnstalling %s (%d/%d), (%d/%d)b - (%d/%d) = %d%% Done\n"),
+					fprintf (stdout, _("Uninstalling %s (%d/%d), (%d/%d)b - (%d/%d) = %d%% Done\n"),
 						 packname,
 						 package_num, num_packages,
 						 total_size_completed, total_size,
@@ -483,104 +475,6 @@ download_failed (EazelInstallCallback *service,
 	}
 }
 
-/* This is ridiculous... */
-static void
-tree_helper_helper(EazelInstallCallback *service,
-		   gchar *indent,
-		   gchar *indent_type,
-		   int indent_level,
-		   GList *iterator,
-		   GList *next_list,
-		   GList **seen)
-{
-	PackageData *pack;
-	char *indent2;
-	char indenter;
-	gchar *extra_space=NULL;
-	int indent_level_cnt;
-
-	if (IS_PACKAGEDATA (iterator->data)) {
-		pack = PACKAGEDATA (iterator->data);
-	} else if (IS_PACKAGEBREAKS (iterator->data)) {
-		PackageBreaks *breakage = PACKAGEBREAKS (iterator->data);
-		pack = packagebreaks_get_package (breakage);
-	} else {
-		PackageDependency *dep = PACKAGEDEPENDENCY (iterator->data);
-		pack = dep->package;
-	}
-
-	if (indent_level>0) {
-		extra_space = g_new0 (char, indent_level+1);
-		for (indent_level_cnt = 0; indent_level_cnt < indent_level; indent_level_cnt++) {
-			extra_space [indent_level_cnt] = ' ';
-		}
-	}
-
-	if (iterator->next || next_list) {
-		indenter = '|';
-	} else {
-		indenter = ' ';
-		*indent_type = '\\';
-	}
-	
-	if (g_list_find (*seen, iterator)==NULL) { 		
-		indent2 = g_strdup_printf ("%s%s%c", indent, extra_space ? extra_space : "", indenter);
-		tree_helper (service, pack, indent2, indent_type, indent_level, NULL, seen);
-		g_free (indent2);
-		(*seen) = g_list_prepend (*seen, iterator);
-	}
-	g_free (extra_space);	
-}
-
-
-static void
-tree_helper (EazelInstallCallback *service,
-	     PackageData *pd,		
-	     gchar *indent,
-	     gchar *indent_type,
-	     int indent_level,
-	     char *title,
-	     GList **seen)
-{
-	char *readable_name;
-	GList *iterator;
-
-	if (title && pd->toplevel) {
-		fprintf (stdout, title);
-	}
-
-	readable_name = packagedata_get_readable_name (pd);
-	fprintf (stdout, "%s%s%s (%s/%s)\n", 
-		 indent,  indent_type,
-		 readable_name,
-		 packagedata_status_enum_to_str (pd->status),
-		 packagedata_modstatus_enum_to_str (pd->modify_status));
-	g_free (readable_name);
-
-	if (g_list_find (*seen, pd)) { return; }
-	(*seen) = g_list_prepend (*seen, pd);
-
-	for (iterator = pd->depends; iterator; iterator = iterator->next) {		
-		char *tmp;
-		tmp = g_strdup ("-d-");
-		tree_helper_helper (service, indent, tmp, indent_level, iterator, 
-				    pd->breaks ? pd->breaks : pd->modifies, seen);
-		g_free (tmp);
-	}
-	for (iterator = pd->breaks; iterator; iterator = iterator->next) {			
-		char *tmp;
-		tmp = g_strdup ("-b-");
-		tree_helper_helper (service, indent, tmp, indent_level, iterator, pd->modifies, seen);
-		g_free (tmp);
-	}
-	for (iterator = pd->modifies; iterator; iterator = iterator->next) {			
-		char *tmp;
-		tmp = g_strdup ("-m-");
-		tree_helper_helper (service, indent, tmp, indent_level, iterator, NULL, seen);
-		g_free (tmp);
-	}
-}
-
 static void
 something_failed (EazelInstallCallback *service,
 		  PackageData *pd,
@@ -604,17 +498,17 @@ something_failed (EazelInstallCallback *service,
 		title = g_strdup_printf (_("Package %s failed to install.\n"), pd->name);
 	}
 
-	if (arg_debug) {
-		GList *seen = NULL;
-		tree_helper (service, pd, "", "", 4, title, &seen);
-		g_list_free (seen);
-		fprintf (stdout, "\n");
-	} else {
-		fprintf (stdout, "%s", title);
-	}
-
+	fprintf (stdout, "%s\n", title);
 	g_free (title);
 
+	if (arg_debug) {
+		GList *list = g_list_prepend (NULL, pd);
+		char *out = packagedata_dump_tree (list, 2);
+		fprintf (stdout, "%s\n", out);
+		g_list_free (list);
+		g_free (out);
+	}
+
 	if (problem) {
 		stuff = eazel_install_problem_tree_to_string (problem, pd, uninstall);
 		if (stuff) {
@@ -623,8 +517,6 @@ something_failed (EazelInstallCallback *service,
 			for (it = stuff; it; it = g_list_next (it)) {
 				/* I18N note: \xB7 is a dot */
 				fprintf (stdout, _("\t\xB7 Problem : %s\n"), (char*)(it->data));
-				/* I18N note: \xB7 is a dot */
-				fprintf (stdout, _("\t\xB7 Problem : %s\n"), (char*)(it->data));
 			}
 			eazel_install_problem_tree_to_case (problem, pd, uninstall, &extra_cases);
 			if (extra_cases) {
@@ -702,10 +594,12 @@ eazel_preflight_check_signal (EazelInstallCallback *service,
 		
 	for (iterator = packages; iterator; iterator = iterator->next) {
 		PackageData *pack = (PackageData*)iterator->data;
-		if (arg_debug ) {
-			GList *seen = NULL;
-			tree_helper (service, pack, "", "", 4, NULL, &seen);
-			g_list_free (seen);
+		if (arg_debug) {
+			GList *list = g_list_prepend (NULL, pack);
+			char *out = packagedata_dump_tree (list, 2);
+			fprintf (stdout, "%s", out);
+			g_list_free (list);
+			g_free (out);
 		} else {
 			char *name = packagedata_get_readable_name (pack);
 			if (pack->depends) {
@@ -722,6 +616,44 @@ eazel_preflight_check_signal (EazelInstallCallback *service,
 	return TRUE;
 }
 
+static gboolean
+eazel_save_transaction_signal (EazelInstallCallback *service, 
+			       EazelInstallCallbackOperation op,
+			       const GList *packages,
+			       gpointer unused) 
+{	
+	char answer[128];
+	gboolean result = FALSE;
+	
+	/* Disabled for 1.0 */
+	return result;
+
+	if (arg_machine) { return TRUE; }
+
+	/* I18N note: the (y/n) is translateable. There is later a 1 character
+	   string with the context "y" which is the "yes" indicator.
+	   If you eg. translate this to Danish : "Fortst (j/n) " and
+	   translated the "y" later to "j", da_DK users can respond with
+	   "j" "ja" "JA" etc. */
+	printf (_("Save transaction report ? (y/n) "));
+	fflush (stdout);
+	if (arg_batch) {			
+		fprintf (stdout, "%s\n", arg_batch);
+		strcpy (answer, arg_batch);
+	} else {
+		fgets (answer, 10, stdin);
+	}
+	/* I18N note: y is the letter for the word Yes. This is
+	   used in the response for a yes/no questions. Your translation
+	   must be 1 character only. */
+	if (strncasecmp (answer, _("y"), 1)==0) {
+		fflush (stdout);
+		result = TRUE;
+	}
+
+	return result;
+}
+
 static void
 dep_check (EazelInstallCallback *service,
 	   const PackageData *package,
@@ -956,6 +888,12 @@ int main(int argc, char *argv[]) {
 	/* Seems that bonobo_main doens't like
 	   not having gnome_init called, dies in a
 	   X call, yech */
+
+#ifdef ENABLE_NLS /* sadly we need this ifdef because otherwise the following get empty statement warnings */
+	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
+	textdomain (PACKAGE);
+#endif
+
 #if 0
 	gnomelib_register_popt_table (oaf_popt_options, oaf_get_popt_table_name ());
 	orb = oaf_init (argc, argv);
@@ -1013,6 +951,9 @@ int main(int argc, char *argv[]) {
 	gtk_signal_connect (GTK_OBJECT (cb), "preflight_check", 
 			    GTK_SIGNAL_FUNC (eazel_preflight_check_signal), 
 			    NULL);
+	gtk_signal_connect (GTK_OBJECT (cb), "save_transaction", 
+			    GTK_SIGNAL_FUNC (eazel_save_transaction_signal), 
+			    NULL);
 	gtk_signal_connect (GTK_OBJECT (cb), "install_progress", 
 			    GTK_SIGNAL_FUNC (eazel_install_progress_signal), 
 			    _("Installing"));
