@@ -47,7 +47,6 @@
 
 #include "eazel-install-metadata.h"
 #include "eazel-install-protocols.h"
-#include "eazel-install-logic.h"
 #include "eazel-install-logic2.h"
 #include "eazel-package-system-types.h"
 
@@ -57,9 +56,15 @@
 #include <dirent.h>
 
 enum {
+	FILE_CONFLICT_CHECK,
+	FILE_UNIQUENESS_CHECK,
+	FEATURE_CONSISTENCY_CHECK,
+
 	DOWNLOAD_PROGRESS,
 	PREFLIGHT_CHECK,
+	SAVE_TRANSACTION,
 	INSTALL_PROGRESS,
+	UNINSTALL_PROGRESS,
 	DOWNLOAD_FAILED,
 	MD5_CHECK_FAILED,
 	INSTALL_FAILED,
@@ -77,7 +82,7 @@ enum {
 	ARG_TEST,	
 	ARG_FORCE,
 	ARG_DEPEND,
-	ARG_UPDATE,
+	ARG_UPGRADE,
 	ARG_UNINSTALL,
 	ARG_DOWNGRADE,
 	ARG_PROTOCOL,
@@ -110,10 +115,18 @@ static BonoboObjectClass *eazel_install_parent_class;
 
 
 /* prototypes */
-
+void eazel_install_emit_file_conflict_check_default (EazelInstall *service, 
+						     const PackageData *package);
+void eazel_install_emit_file_uniqueness_check_default (EazelInstall *service, 
+						       const PackageData *package);
+void eazel_install_emit_feature_consistency_check_default (EazelInstall *service, 
+							   const PackageData *package);
 void eazel_install_emit_install_progress_default (EazelInstall *service,
 						  const PackageData *pack,
 						  int, int, int, int, int, int);
+void eazel_install_emit_uninstall_progress_default (EazelInstall *service,
+						    const PackageData *pack,
+						    int, int, int, int, int, int);
 void  eazel_install_emit_download_progress_default (EazelInstall *service,
 						    const PackageData *package,
 						    int amount,
@@ -122,6 +135,8 @@ gboolean  eazel_install_emit_preflight_check_default (EazelInstall *service,
 						      GList *packages,
 						      int total_bytes,
 						      int total_packages);
+gboolean  eazel_install_emit_save_transaction_default (EazelInstall *service, 
+						      GList *packages);
 void  eazel_install_emit_download_failed_default (EazelInstall *service,
 						  const PackageData *package);
 void eazel_install_emit_md5_check_failed_default (EazelInstall *service,
@@ -152,69 +167,6 @@ static void eazel_install_log (const char *domain,
 
 static PortableServer_ServantBase__epv base_epv = { NULL, NULL, NULL };
 
-static CORBA_char*
-xml_from_packagedata (const PackageData *pack) {
-	xmlDocPtr doc;
-	xmlNodePtr node;
-	xmlChar *mem;
-	CORBA_char *result;
-	int size;
-
-	doc = xmlNewDoc ("1.0");
-
-	node = xmlNewDocNode (doc, NULL, "CATEGORIES", NULL);
-	xmlDocSetRootElement (doc, node);
-	node = xmlAddChild (node, xmlNewNode (NULL, "CATEGORY"));
-	xmlSetProp (node, "name", "failed");
-	
-	node = xmlAddChild (node, xmlNewNode (NULL, "PACKAGES"));
-
-	xmlAddChild (node, 
-		     eazel_install_packagedata_to_xml (pack, NULL, NULL, TRUE));
-
-	xmlDocDumpMemory (doc, &mem, &size);
-	result = CORBA_string_dup (mem);
-	free (mem);
-	xmlFreeDoc (doc);
-
-	return result;
-}
-
-static CORBA_char*
-xml_from_packagedata_list (const GList *packages) {
-	xmlDocPtr doc;
-	xmlNodePtr node;
-	xmlChar *mem;
-	CORBA_char *result;
-	int size;
-	const GList *iterator;
-
-	doc = xmlNewDoc ("1.0");
-
-	node = xmlNewDocNode (doc, NULL, "CATEGORIES", NULL);
-	xmlDocSetRootElement (doc, node);
-	node = xmlAddChild (node, xmlNewNode (NULL, "CATEGORY"));
-	xmlSetProp (node, "name", "failed");
-	
-	node = xmlAddChild (node, xmlNewNode (NULL, "PACKAGES"));
-
-	for (iterator = packages; iterator; glist_step (iterator)) {
-		PackageData *pack = (PackageData*)iterator->data;
-		xmlAddChild (node, 
-			     eazel_install_packagedata_to_xml (pack, 
-							       NULL, 
-							       NULL, 
-							       TRUE));
-	}
-
-	xmlDocDumpMemory (doc, &mem, &size);
-	result = CORBA_string_dup (mem);
-	free (mem);
-	xmlFreeDoc (doc);
-
-	return result;
-}
-
 #endif /* EAZEL_INSTALL_NO_CORBA */
 
 /*****************************************
@@ -254,6 +206,7 @@ eazel_install_finalize (GtkObject *object)
 	g_list_free (service->private->root_dirs);
 
 	g_list_free (service->private->transaction);
+	g_list_free (service->private->failed_packages);
 	
 	g_free (service->private->transaction_dir);
 	g_free (service->private->cur_root);
@@ -280,6 +233,120 @@ eazel_install_unref (GtkObject *object)
 #endif
 }
 
+static gboolean
+eazel_install_start_signal (EazelPackageSystem *system,
+			    EazelPackageSystemOperation op,
+			    const PackageData *pack,
+			    EazelInstall *service)
+{
+	service->private->infoblock[2]++;
+	switch (op) {
+	case EAZEL_PACKAGE_SYSTEM_OPERATION_INSTALL:
+		eazel_install_emit_install_progress (service, 
+						     pack,
+						     service->private->infoblock[2], service->private->infoblock[3],
+						     0, pack->bytesize,
+						     service->private->infoblock[4], service->private->infoblock[5]);				     
+		break;
+	case EAZEL_PACKAGE_SYSTEM_OPERATION_UNINSTALL:
+		eazel_install_emit_uninstall_progress (service, 
+						       pack,
+						       service->private->infoblock[2], service->private->infoblock[3],
+						       0, pack->bytesize,
+						       service->private->infoblock[4], service->private->infoblock[5]);				     
+		break;
+	default:
+		break;
+	}
+	return TRUE;
+}
+
+static gboolean
+eazel_install_end_signal (EazelPackageSystem *system,
+			  EazelPackageSystemOperation op,
+			  const PackageData *pack,
+			  EazelInstall *service)
+{
+	switch (op) {
+	case EAZEL_PACKAGE_SYSTEM_OPERATION_INSTALL:
+		eazel_install_emit_install_progress (service, 
+						     pack,
+						     service->private->infoblock[2], service->private->infoblock[3],
+						     pack->bytesize, pack->bytesize,
+						     service->private->infoblock[4], service->private->infoblock[5]);				     
+		break;
+	case EAZEL_PACKAGE_SYSTEM_OPERATION_UNINSTALL:
+		eazel_install_emit_uninstall_progress (service, 
+						       pack,
+						       service->private->infoblock[2], service->private->infoblock[3],
+						       pack->bytesize, pack->bytesize,
+						       service->private->infoblock[4], service->private->infoblock[5]);				     
+		break;
+	default:
+		break;
+	}
+	if (pack->toplevel) {
+		if (g_list_find (service->private->failed_packages, (PackageData*)pack) == NULL) {
+			g_message ("Adding %s to transaction", pack->name);
+			service->private->transaction = g_list_prepend (service->private->transaction,
+									(PackageData*)pack);
+		}
+	}
+	return TRUE;
+}
+
+static gboolean
+eazel_install_progress_signal (EazelPackageSystem *system,
+			       EazelPackageSystemOperation op,
+			       const PackageData *pack,
+			       unsigned long *info,
+			       EazelInstall *service)
+{
+	service->private->infoblock[4] = info[4];
+	if ((info[0] != 0) && (info[0] != info[1])) {
+		switch (op) {
+		case EAZEL_PACKAGE_SYSTEM_OPERATION_INSTALL:
+			eazel_install_emit_install_progress (service, 
+							     pack,
+							     service->private->infoblock[2], service->private->infoblock[3],
+							     info[0], pack->bytesize,
+							     info[4], info[5]);
+			break;
+		case EAZEL_PACKAGE_SYSTEM_OPERATION_UNINSTALL:
+			eazel_install_emit_uninstall_progress (service, 
+							       pack,
+							       service->private->infoblock[2], service->private->infoblock[3],
+							       1, pack->bytesize,
+							       info[4], info[5]);
+			break;
+		default:
+			break;
+		}
+	}
+	return TRUE;
+}
+
+static gboolean
+eazel_install_failed_signal (EazelPackageSystem *system,
+			     EazelPackageSystemOperation op,
+			     const PackageData *pack,
+			     EazelInstall *service)
+{
+	trilobite_debug ("*** %s failed", pack->name);
+
+	service->private->failed_packages = g_list_prepend (service->private->failed_packages, 
+							    (PackageData*)pack);
+
+	if (pack->toplevel) {
+		trilobite_debug ("emiting failed for %s", pack->name);
+		if (op==EAZEL_PACKAGE_SYSTEM_OPERATION_INSTALL) {
+			eazel_install_emit_install_failed (service, pack);
+		} else if (op==EAZEL_PACKAGE_SYSTEM_OPERATION_UNINSTALL) {
+			eazel_install_emit_uninstall_failed (service, pack);
+		}
+	}
+	return TRUE;
+}
 
 static void
 eazel_install_set_arg (GtkObject *object,
@@ -321,8 +388,8 @@ eazel_install_set_arg (GtkObject *object,
 	case ARG_DEPEND:
 		eazel_install_set_depend (service, GTK_VALUE_BOOL(*arg));
 		break;
-	case ARG_UPDATE:
-		eazel_install_set_update (service, GTK_VALUE_BOOL(*arg));
+	case ARG_UPGRADE:
+		eazel_install_set_upgrade (service, GTK_VALUE_BOOL(*arg));
 		break;
 	case ARG_UNINSTALL:
 		eazel_install_set_uninstall (service, GTK_VALUE_BOOL(*arg));
@@ -385,6 +452,28 @@ eazel_install_class_initialize (EazelInstallClass *klass)
 	((POA_GNOME_Trilobite_Eazel_Install__vepv*)klass->servant_vepv)->GNOME_Trilobite_Eazel_Install_epv = eazel_install_get_epv ();
 #endif /* EAZEL_INSTALL_NO_CORBA */
 
+	signals[FILE_CONFLICT_CHECK] = 
+		gtk_signal_new ("file_conflict_check",
+				GTK_RUN_LAST,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (EazelInstallClass, file_conflict_check),
+				gtk_marshal_NONE__POINTER,
+				GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
+	signals[FILE_UNIQUENESS_CHECK] = 
+		gtk_signal_new ("file_uniqueness_check",
+				GTK_RUN_LAST,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (EazelInstallClass, file_uniqueness_check),
+				gtk_marshal_NONE__POINTER,
+				GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
+	signals[FEATURE_CONSISTENCY_CHECK] = 
+		gtk_signal_new ("feature_consistency_check",
+				GTK_RUN_LAST,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (EazelInstallClass, feature_consistency_check),
+				gtk_marshal_NONE__POINTER,
+				GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
+
 	signals[DOWNLOAD_PROGRESS] = 
 		gtk_signal_new ("download_progress",
 				GTK_RUN_LAST,
@@ -399,6 +488,13 @@ eazel_install_class_initialize (EazelInstallClass *klass)
 				GTK_SIGNAL_OFFSET (EazelInstallClass, preflight_check),
 				gtk_marshal_BOOL__POINTER_INT_INT,
 				GTK_TYPE_BOOL, 3, GTK_TYPE_POINTER, GTK_TYPE_INT, GTK_TYPE_INT);
+	signals[SAVE_TRANSACTION] = 
+		gtk_signal_new ("save_transaction",
+				GTK_RUN_LAST,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (EazelInstallClass, save_transaction),
+				gtk_marshal_BOOL__POINTER,
+				GTK_TYPE_BOOL, 1, GTK_TYPE_POINTER);
 	signals[INSTALL_PROGRESS] = 
 		gtk_signal_new ("install_progress",
 				GTK_RUN_LAST,
@@ -408,6 +504,15 @@ eazel_install_class_initialize (EazelInstallClass *klass)
 				GTK_TYPE_NONE, 7, GTK_TYPE_POINTER, 
 				GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_INT, 
 				GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_INT);
+	signals[UNINSTALL_PROGRESS] = 
+		gtk_signal_new ("uninstall_progress",
+				GTK_RUN_LAST,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (EazelInstallClass, uninstall_progress),
+				eazel_install_gtk_marshal_NONE__POINTER_INT_INT_INT_INT_INT_INT,
+				GTK_TYPE_NONE, 7, GTK_TYPE_POINTER, 
+				GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_INT, 
+				GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_INT);
 	signals[DOWNLOAD_FAILED] = 
 		gtk_signal_new ("download_failed",
 				GTK_RUN_LAST,
@@ -453,7 +558,11 @@ eazel_install_class_initialize (EazelInstallClass *klass)
 	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
 
 #ifdef EAZEL_INSTALL_NO_CORBA
+	klass->file_conflict_check = NULL;
+	klass->file_uniqueness_check = NULL;
+	klass->feature_consistency_check = NULL;
 	klass->install_progress = NULL;
+	klass->uninstall_progress = NULL;
 	klass->download_progress = NULL;
 	klass->download_failed = NULL;
 	klass->md5_check_failed = NULL;
@@ -461,8 +570,13 @@ eazel_install_class_initialize (EazelInstallClass *klass)
 	klass->uninstall_failed = NULL;
 	klass->dependency_check = NULL;
 	klass->preflight_check = NULL;
+	klass->save_transaction = NULL;
 #else
+	klass->file_conflict_check = eazel_install_emit_file_conflict_check_default;
+	klass->file_uniqueness_check = eazel_install_emit_file_uniqueness_check_default;
+	klass->feature_consistency_check = eazel_install_emit_feature_consistency_check_default;
 	klass->install_progress = eazel_install_emit_install_progress_default;
+	klass->uninstall_progress = eazel_install_emit_uninstall_progress_default;
 	klass->download_progress = eazel_install_emit_download_progress_default;
 	klass->download_failed = eazel_install_emit_download_failed_default;
 	klass->md5_check_failed = eazel_install_emit_md5_check_failed_default;
@@ -470,6 +584,7 @@ eazel_install_class_initialize (EazelInstallClass *klass)
 	klass->uninstall_failed = eazel_install_emit_uninstall_failed_default;
 	klass->dependency_check = eazel_install_emit_dependency_check_default;
 	klass->preflight_check = eazel_install_emit_preflight_check_default;
+	klass->save_transaction = eazel_install_emit_save_transaction_default;
 #endif
 	klass->done = eazel_install_emit_done_default;
 
@@ -505,10 +620,10 @@ eazel_install_class_initialize (EazelInstallClass *klass)
 				 GTK_TYPE_BOOL,
 				 GTK_ARG_READWRITE,
 				 ARG_DEPEND);
-	gtk_object_add_arg_type ("EazelInstall::update",
+	gtk_object_add_arg_type ("EazelInstall::upgrade",
 				 GTK_TYPE_BOOL,
 				 GTK_ARG_READWRITE,
-				 ARG_UPDATE);
+				 ARG_UPGRADE);
 	gtk_object_add_arg_type ("EazelInstall::uninstall",
 				 GTK_TYPE_BOOL,
 				 GTK_ARG_READWRITE,
@@ -569,6 +684,7 @@ eazel_install_class_initialize (EazelInstallClass *klass)
 
 static void
 eazel_install_initialize (EazelInstall *service) {
+	GList *list;
 #ifndef EAZEL_INSTALL_NO_CORBA
 	GNOME_Trilobite_Eazel_Install corba_service;
 #endif /* EAZEL_INSTALL_NO_CORBA */
@@ -607,6 +723,7 @@ eazel_install_initialize (EazelInstall *service) {
 							  (GCompareFunc)g_str_equal);
 	service->private->downloaded_files = NULL;
 	service->private->transaction = NULL;
+	service->private->failed_packages = NULL;
 	service->private->revert = FALSE;
 	service->private->ssl_rename = FALSE;
 	service->private->ignore_file_conflicts = FALSE;
@@ -623,38 +740,36 @@ eazel_install_initialize (EazelInstall *service) {
 
 	trilobite_debug (_("Transactions are stored in %s"), service->private->transaction_dir);
 
-	{
-		GList *list = NULL;
+	list = NULL;
+#ifndef EAZEL_INSTALL_SLIM
+	if (eazel_install_configure_use_local_db ()) {
 		char *tmp = NULL;
 
-#ifndef EAZEL_INSTALL_SLIM
-		tmp = g_strdup_printf ("%s/.nautilus/rpmdb/", g_get_home_dir ());
+		tmp = g_strdup_printf ("%s/.nautilus/packagedb/", g_get_home_dir ());
 		list = g_list_prepend (list, g_strdup (g_get_home_dir ()));
 		list = g_list_prepend (list, tmp);
-#else
-		tmp = tmp;
+	}
 #endif
 
-		service->private->package_system = eazel_package_system_new (list);
-		eazel_package_system_set_debug (service->private->package_system, 
-						EAZEL_PACKAGE_SYSTEM_DEBUG_FAIL);
-		gtk_signal_connect (GTK_OBJECT (service->private->package_system),
-				    "start",
-				    (GtkSignalFunc)eazel_install_start_signal,
-				    service);
-		gtk_signal_connect (GTK_OBJECT (service->private->package_system),
-				    "progress",
-				    (GtkSignalFunc)eazel_install_progress_signal,
-				    service);
-		gtk_signal_connect (GTK_OBJECT (service->private->package_system),
-				    "failed",
-				    (GtkSignalFunc)eazel_install_failed_signal,
-				    service);
-		gtk_signal_connect (GTK_OBJECT (service->private->package_system),
-				    "end",
-				    (GtkSignalFunc)eazel_install_end_signal,
-				    service);
-	}
+	service->private->package_system = eazel_package_system_new (list);
+	eazel_package_system_set_debug (service->private->package_system, 
+					EAZEL_PACKAGE_SYSTEM_DEBUG_FAIL);
+	gtk_signal_connect (GTK_OBJECT (service->private->package_system),
+			    "start",
+			    (GtkSignalFunc)eazel_install_start_signal,
+			    service);
+	gtk_signal_connect (GTK_OBJECT (service->private->package_system),
+			    "progress",
+			    (GtkSignalFunc)eazel_install_progress_signal,
+			    service);
+	gtk_signal_connect (GTK_OBJECT (service->private->package_system),
+			    "failed",
+			    (GtkSignalFunc)eazel_install_failed_signal,
+			    service);
+	gtk_signal_connect (GTK_OBJECT (service->private->package_system),
+			    "end",
+			    (GtkSignalFunc)eazel_install_end_signal,
+			    service);
 }
 
 GtkType
@@ -719,7 +834,7 @@ eazel_install_new_with_config (void)
 						 "test", iopts->mode_test,
 						 "force", iopts->mode_force,
 						 "depend", iopts->mode_depend,
-						 "update", iopts->mode_update,
+						 "upgrade", iopts->mode_update,
 						 "uninstall", iopts->mode_uninstall,
 						 "downgrade", iopts->mode_downgrade,
 						 "protocol", iopts->protocol,
@@ -739,6 +854,11 @@ eazel_install_new_with_config (void)
 	return service;
 }
 
+
+/*****************************************
+  install stuff
+*****************************************/
+
 /* if there's an older tmpdir left over from a previous attempt, use it */
 static char *
 find_old_tmpdir (const char *prefix)
@@ -1067,6 +1187,10 @@ eazel_install_install_packages (EazelInstall *service,
 	trilobite_debug ("eazel_install_install_packages (..., %d cats, %s)", 
 			 g_list_length (categories),
 			 root);
+	trilobite_debug ("eazel_install_install_packages (upgrade = %d, downgrade = %d, force = %d)",
+			 eazel_install_get_upgrade (service),
+			 eazel_install_get_downgrade (service),
+			 eazel_install_get_force (service));
 
 #ifndef EAZEL_INSTALL_SLIM
 	/* we're about to call g_main_iteraton sometimes, so grab a ref on ourself to avoid vanishing. */
@@ -1094,8 +1218,7 @@ eazel_install_install_packages (EazelInstall *service,
 			g_warning (_("Install failed"));
 		} 
 		
-		eazel_install_unlock_tmp_dir (service);
-		trilobite_debug ("service->private->downloaded_files = 0x%p", 
+		trilobite_debug ("service->private->downloaded_files = %p", 
 				 (unsigned int)service->private->downloaded_files);
 		
 		g_free (service->private->cur_root);
@@ -1104,11 +1227,11 @@ eazel_install_install_packages (EazelInstall *service,
 		result = EAZEL_INSTALL_NOTHING;
 	}
 
+	eazel_install_emit_done (service, result & EAZEL_INSTALL_INSTALL_OK);
+		
 #ifndef EAZEL_INSTALL_SLIM
 	bonobo_object_unref (BONOBO_OBJECT (service));
 #endif /* EAZEL_INSTALL_SLIM */
-
-	eazel_install_emit_done (service, result & EAZEL_INSTALL_INSTALL_OK);
 }
 
 void 
@@ -1135,19 +1258,12 @@ eazel_install_uninstall_packages (EazelInstall *service, GList *categories, cons
 		eazel_install_fetch_remote_package_list (service);
 	}
 
-/*
-  FIXME: bugzilla.eazel.com 5752
-	if (eazel_install_get_ei2 (service)) {
-		result = uninstall_packages (service, categories);
-	} else {
-		result = ei_uninstall_packages (service, categories);
-	}
-*/
-	result = ei_uninstall_packages (service, categories);
+	result = uninstall_packages (service, categories);
 
 	if (result == EAZEL_INSTALL_NOTHING) {
 		g_warning (_("Uninstall failed"));
 	} 
+
 	eazel_install_emit_done (service, result & EAZEL_INSTALL_UNINSTALL_OK);
 }
 
@@ -1168,21 +1284,12 @@ eazel_install_revert_transaction_from_xmlstring (EazelInstall *service,
 	service->private->revert = TRUE;
 
 	if (create_temporary_directory (service)) {
-
-/*
-  FIXME: bugzilla.eazel.com 5753
-		if (eazel_install_get_ei2 (service)) {
-			result = revert_transaction (service, packages);
-		} else {
-			result = ei_revert_transaction (service, packages);
-		}
-*/
-		result = ei_revert_transaction (service, packages);
-		eazel_install_unlock_tmp_dir (service);
+		result = revert_transaction (service, packages);
 	} else {
 		result = EAZEL_INSTALL_NOTHING;
 	} 
 	service->private->revert = FALSE;
+
 	eazel_install_emit_done (service, result & EAZEL_INSTALL_REVERSION_OK);
 }
 
@@ -1229,6 +1336,110 @@ eazel_install_add_repository (EazelInstall *service, const char *dir)
 	service->private->local_repositories = g_list_prepend (service->private->local_repositories, g_strdup (dir));
 }
 
+static void
+eazel_install_do_transaction_save_report_helper (xmlNodePtr node,
+						     GList *packages)
+{
+	GList *iterator;
+
+	for (iterator = packages; iterator; iterator = g_list_next (iterator)) {
+		PackageData *pack;
+		char *tmp;
+		pack = (PackageData*)iterator->data;
+		switch (pack->modify_status) {
+		case PACKAGE_MOD_INSTALLED:			
+			tmp = g_strdup_printf ("Installed %s", pack->name);
+			xmlNewChild (node, NULL, "DESCRIPTION", tmp);
+			g_free (tmp);
+			break;
+		case PACKAGE_MOD_UNINSTALLED:			
+			tmp = g_strdup_printf ("Uninstalled %s", pack->name);
+			xmlNewChild (node, NULL, "DESCRIPTION", tmp);
+			g_free (tmp);
+			break;
+		default:
+			break;
+		}
+		if (pack->modifies) {
+			eazel_install_do_transaction_save_report_helper (node, pack->modifies);
+		}
+	}
+}
+
+void
+eazel_install_save_transaction_report (EazelInstall *service) 
+{
+	FILE *outfile;
+	xmlDocPtr doc;
+	xmlNodePtr node, root;
+	char *name = NULL;
+
+	if (eazel_install_get_transaction_dir (service) == NULL) {
+		g_warning ("Transaction directory not set, not storing transaction report");
+	}
+
+	/* Ensure the transaction dir is present */
+	if (! g_file_test (eazel_install_get_transaction_dir (service), G_FILE_TEST_ISDIR)) {
+		int retval;
+		retval = mkdir (eazel_install_get_transaction_dir (service), 0755);		       
+		if (retval < 0) {
+			if (errno != EEXIST) {
+				g_warning (_("Could not create transaction directory (%s)! ***\n"), 
+					   eazel_install_get_transaction_dir (service));
+				return;
+			}
+		}
+	}
+
+	/* Create xml */
+	doc = xmlNewDoc ("1.0");
+	root = node = xmlNewNode (NULL, "TRANSACTION");
+	xmlDocSetRootElement (doc, node);
+
+	/* Make a unique name */
+	name = g_strdup_printf ("%s/transaction.%lu", eazel_install_get_transaction_dir (service),
+				(unsigned long) time (NULL));
+	while (g_file_test (name, G_FILE_TEST_ISFILE)) {
+		g_free (name);
+		sleep (1);
+		name = g_strdup_printf ("%s/transaction.%lu", 
+					eazel_install_get_transaction_dir (service), 
+					(unsigned long) time (NULL));
+	}
+
+	trilobite_debug (_("Writing transaction to %s"), name);
+	
+	/* Open and save */
+	outfile = fopen (name, "w");
+	xmlAddChild (root, eazel_install_packagelist_to_xml (service->private->transaction, FALSE));
+	node = xmlAddChild (node, xmlNewNode (NULL, "DESCRIPTIONS"));
+
+	{
+		char *tmp;
+		tmp = g_strdup_printf ("%lu", (unsigned long) time (NULL));		
+		xmlNewChild (node, NULL, "DATE", tmp);
+		g_free (tmp);
+	}
+
+	eazel_install_do_transaction_save_report_helper (node, service->private->transaction);
+
+	xmlDocDump (outfile, doc);
+	
+	fclose (outfile);
+	g_free (name);
+}
+
+void
+eazel_install_init_transaction (EazelInstall *service) 
+{
+	/* Null the list of files met in the transaction */
+	g_list_free (service->private->transaction);
+	service->private->transaction = NULL;
+
+	/* Null the list of files met in the transaction */
+	g_list_free (service->private->failed_packages);
+	service->private->failed_packages = NULL;
+}
 
 /************************************************
   Signal emitters and default handlers.
@@ -1236,6 +1447,90 @@ eazel_install_add_repository (EazelInstall *service, const char *dir)
   a corba callback, and if true, do the callback
 **************************************************/
 					 
+void 
+eazel_install_emit_file_conflict_check (EazelInstall *service, 
+					const PackageData *pack)
+{
+	EAZEL_INSTALL_SANITY(service);
+	gtk_signal_emit (GTK_OBJECT (service), signals[FILE_CONFLICT_CHECK], pack);
+}
+
+void 
+eazel_install_emit_file_conflict_check_default (EazelInstall *service, 
+						const PackageData *pack)
+{
+#ifndef EAZEL_INSTALL_NO_CORBA
+	CORBA_Environment ev;
+	CORBA_exception_init (&ev);
+	EAZEL_INSTALL_SANITY(service);
+	if (service->callback != CORBA_OBJECT_NIL) {
+		GNOME_Trilobite_Eazel_PackageDataStruct *package;
+		package = corba_packagedatastruct_from_packagedata (pack);
+		GNOME_Trilobite_Eazel_InstallCallback_file_conflict_check (service->callback, 
+									   package, 
+									   &ev);
+		CORBA_free (package);
+	} 
+	CORBA_exception_free (&ev);
+#endif /* EAZEL_INSTALL_NO_CORBA */
+}
+
+void 
+eazel_install_emit_file_uniqueness_check (EazelInstall *service, 
+					const PackageData *pack)
+{
+	EAZEL_INSTALL_SANITY(service);
+	gtk_signal_emit (GTK_OBJECT (service), signals[FILE_UNIQUENESS_CHECK], pack);
+}
+
+void 
+eazel_install_emit_file_uniqueness_check_default (EazelInstall *service, 
+						  const PackageData *pack)
+{
+#ifndef EAZEL_INSTALL_NO_CORBA
+	CORBA_Environment ev;
+	CORBA_exception_init (&ev);
+	EAZEL_INSTALL_SANITY(service);
+	if (service->callback != CORBA_OBJECT_NIL) {
+		GNOME_Trilobite_Eazel_PackageDataStruct *package;
+		package = corba_packagedatastruct_from_packagedata (pack);
+		GNOME_Trilobite_Eazel_InstallCallback_file_uniqueness_check (service->callback, 
+									     package, 
+									     &ev);
+		CORBA_free (package);
+	} 
+	CORBA_exception_free (&ev);
+#endif /* EAZEL_INSTALL_NO_CORBA */
+}
+
+void 
+eazel_install_emit_feature_consistency_check (EazelInstall *service, 
+					      const PackageData *pack)
+{
+	EAZEL_INSTALL_SANITY(service);
+	gtk_signal_emit (GTK_OBJECT (service), signals[FEATURE_CONSISTENCY_CHECK], pack);
+}
+
+
+void 
+eazel_install_emit_feature_consistency_check_default (EazelInstall *service, 
+						      const PackageData *pack)
+{
+#ifndef EAZEL_INSTALL_NO_CORBA
+	CORBA_Environment ev;
+	CORBA_exception_init (&ev);
+	EAZEL_INSTALL_SANITY(service);
+	if (service->callback != CORBA_OBJECT_NIL) {
+		GNOME_Trilobite_Eazel_PackageDataStruct *package;
+		package = corba_packagedatastruct_from_packagedata (pack);
+		GNOME_Trilobite_Eazel_InstallCallback_feature_consistency_check (service->callback, 
+										 package, 
+										 &ev);
+		CORBA_free (package);
+	} 
+	CORBA_exception_free (&ev);
+#endif /* EAZEL_INSTALL_NO_CORBA */
+}
 
 void 
 eazel_install_emit_install_progress (EazelInstall *service, 
@@ -1279,6 +1574,49 @@ eazel_install_emit_install_progress_default (EazelInstall *service,
 #endif /* EAZEL_INSTALL_NO_CORBA */
 } 
 
+void 
+eazel_install_emit_uninstall_progress (EazelInstall *service, 
+				       const PackageData *pack,
+				       int package_num, int num_packages, 
+				       int package_size_completed, int package_size_total,
+				       int total_size_completed, int total_size)
+{
+	EAZEL_INSTALL_SANITY(service);
+	gtk_signal_emit (GTK_OBJECT (service), signals[UNINSTALL_PROGRESS], 
+			 pack,
+			 package_num, num_packages,
+			 package_size_completed, package_size_total,
+			 total_size_completed, total_size);
+
+}
+
+void 
+eazel_install_emit_uninstall_progress_default (EazelInstall *service, 
+					       const PackageData *pack,
+					       int package_num, int num_packages, 
+					       int package_size_completed, int package_size_total,
+					       int total_size_completed, int total_size)
+{
+#ifndef EAZEL_INSTALL_NO_CORBA
+	CORBA_Environment ev;
+	CORBA_exception_init (&ev);
+	EAZEL_INSTALL_SANITY(service);
+
+	if (service->callback != CORBA_OBJECT_NIL) {
+		GNOME_Trilobite_Eazel_PackageDataStruct *package;
+		package = corba_packagedatastruct_from_packagedata (pack);
+		GNOME_Trilobite_Eazel_InstallCallback_uninstall_progress (service->callback, 
+									  package, 
+									  package_num, num_packages,
+									  package_size_completed, package_size_total,
+									  total_size_completed, total_size,
+									  &ev);
+		CORBA_free (package);
+	} 
+	CORBA_exception_free (&ev);
+#endif /* EAZEL_INSTALL_NO_CORBA */
+} 
+
 void 
 eazel_install_emit_download_progress (EazelInstall *service, 
 				      const PackageData *pack,
@@ -1338,8 +1676,6 @@ gboolean
 eazel_install_emit_preflight_check (EazelInstall *service, 
 				    GList *packages)
 {
-	GList *packages_in_signal = NULL;
-	GList *iterator;
 	unsigned long size_packages, num_packages;
 	gboolean result;
 	GList *flat_packages;
@@ -1351,21 +1687,12 @@ eazel_install_emit_preflight_check (EazelInstall *service,
 	num_packages = g_list_length (flat_packages);
 	g_list_free (flat_packages);
 
-	for (iterator = packages; iterator; glist_step (iterator)) {
-		PackageData *pack = (PackageData*)iterator->data;
-		
-		if (eazel_install_get_ei2 (service) || pack->toplevel) {
-			packages_in_signal = g_list_prepend (packages_in_signal, pack);
-		} 
-	}
-
 	gtk_signal_emit (GTK_OBJECT (service), 
 			 signals[PREFLIGHT_CHECK], 
-			 packages_in_signal,
+			 packages,
 			 size_packages,
 			 num_packages,
 			 &result);
-	g_list_free (packages_in_signal);
 
 	return result;
 }
@@ -1379,24 +1706,33 @@ eazel_install_emit_preflight_check_default (EazelInstall *service,
 #ifndef EAZEL_INSTALL_NO_CORBA
 	CORBA_Environment ev;
 	CORBA_boolean result = FALSE;
+	GNOME_Trilobite_Eazel_PackageDataStructList *package_tree;
+	GNOME_Trilobite_Eazel_Operation corba_op;
 
 	CORBA_exception_init (&ev);
 	EAZEL_INSTALL_SANITY_VAL (service, FALSE);
 
-	if (service->callback != CORBA_OBJECT_NIL) {
-		CORBA_char *corbapackages;
-		corbapackages = xml_from_packagedata_list (packages);
+	if (service->private->revert) {
+		corba_op = GNOME_Trilobite_Eazel_OPERATION_REVERT;
+	} else if (eazel_install_get_uninstall (service)==TRUE) {
+		corba_op = GNOME_Trilobite_Eazel_OPERATION_UNINSTALL;		
+	} else {
+		corba_op = GNOME_Trilobite_Eazel_OPERATION_INSTALL;
+	} 
 
+	if (service->callback != CORBA_OBJECT_NIL) {
+		package_tree = corba_packagedatastructlist_from_packagedata_tree (packages);
 		result = GNOME_Trilobite_Eazel_InstallCallback_preflight_check (service->callback, 
-									  corbapackages,
-									  total_bytes,
-									  total_packages, 
-									  &ev);
+										corba_op,
+										package_tree,
+										total_bytes,
+										total_packages, 
+										&ev);
 		if (ev._major != CORBA_NO_EXCEPTION) {
 			/* abort on corba failure */
 			result = FALSE;
 		}
-		CORBA_free (corbapackages);
+		CORBA_free (package_tree);
 	} 
 	CORBA_exception_free (&ev);
 	return (gboolean)result;
@@ -1405,6 +1741,62 @@ eazel_install_emit_preflight_check_default (EazelInstall *service,
 #endif /* EAZEL_INSTALL_NO_CORBA */
 } 
 
+gboolean
+eazel_install_emit_save_transaction (EazelInstall *service, 
+				     GList *packages)
+{
+	gboolean result;
+
+	EAZEL_INSTALL_SANITY_VAL(service, FALSE);
+
+	gtk_signal_emit (GTK_OBJECT (service), 
+			 signals[SAVE_TRANSACTION], 
+			 packages,
+			 &result);
+
+	return result;
+}
+
+gboolean
+eazel_install_emit_save_transaction_default (EazelInstall *service, 
+					     GList *packages)
+{
+#ifndef EAZEL_INSTALL_NO_CORBA
+	CORBA_Environment ev;
+	CORBA_boolean result = FALSE;
+	GNOME_Trilobite_Eazel_PackageDataStructList *package_tree;
+	GNOME_Trilobite_Eazel_Operation corba_op;
+
+	CORBA_exception_init (&ev);
+	EAZEL_INSTALL_SANITY_VAL (service, FALSE);
+
+	if (service->private->revert) {
+		corba_op = GNOME_Trilobite_Eazel_OPERATION_REVERT;
+	} else if (eazel_install_get_uninstall (service)==TRUE) {
+		corba_op = GNOME_Trilobite_Eazel_OPERATION_UNINSTALL;		
+	} else {
+		corba_op = GNOME_Trilobite_Eazel_OPERATION_INSTALL;
+	} 
+
+	if (service->callback != CORBA_OBJECT_NIL) {
+		package_tree = corba_packagedatastructlist_from_packagedata_tree (packages);
+		result = GNOME_Trilobite_Eazel_InstallCallback_save_transaction (service->callback, 
+										 corba_op,
+										 package_tree,
+										 &ev);
+		if (ev._major != CORBA_NO_EXCEPTION) {
+			/* abort on corba failure */
+			result = FALSE;
+		}
+		CORBA_free (package_tree);
+	} 
+	CORBA_exception_free (&ev);
+	return (gboolean)result;
+#else /* EAZEL_INSTALL_NO_CORBA */
+	return FALSE;
+#endif /* EAZEL_INSTALL_NO_CORBA */
+} 
+
 void 
 eazel_install_emit_download_failed (EazelInstall *service, 
 				    const PackageData *pack)
@@ -1482,13 +1874,18 @@ eazel_install_emit_install_failed_default (EazelInstall *service,
 {
 #ifndef EAZEL_INSTALL_NO_CORBA
 	CORBA_Environment ev;
+	GNOME_Trilobite_Eazel_PackageDataStructList *package_tree;
+	GList *list;
+
+	EAZEL_INSTALL_SANITY (service);
 	CORBA_exception_init (&ev);
-	EAZEL_INSTALL_SANITY(service);
 	if (service->callback != CORBA_OBJECT_NIL) {
-		CORBA_char *package;
-		package = xml_from_packagedata (pack);
-		GNOME_Trilobite_Eazel_InstallCallback_install_failed (service->callback, package, &ev);	
-		CORBA_free (package);
+		list = g_list_prepend (NULL, (void *)pack);
+		package_tree = corba_packagedatastructlist_from_packagedata_tree (list);
+		g_list_free (list);
+
+		GNOME_Trilobite_Eazel_InstallCallback_install_failed (service->callback, package_tree, &ev);
+		CORBA_free (package_tree);
 	} 
 	CORBA_exception_free (&ev);
 #endif /* EAZEL_INSTALL_NO_CORBA */
@@ -1508,13 +1905,18 @@ eazel_install_emit_uninstall_failed_default (EazelInstall *service,
 {
 #ifndef EAZEL_INSTALL_NO_CORBA
 	CORBA_Environment ev;
+	GNOME_Trilobite_Eazel_PackageDataStructList *package_tree;
+	GList *list;
+
+	EAZEL_INSTALL_SANITY (service);
 	CORBA_exception_init (&ev);
-	EAZEL_INSTALL_SANITY(service);
 	if (service->callback != CORBA_OBJECT_NIL) {
-		CORBA_char *package;
-		package = xml_from_packagedata (pack);
-		GNOME_Trilobite_Eazel_InstallCallback_uninstall_failed (service->callback, package, &ev);	
-		/* CORBA_free (package); */
+		list = g_list_prepend (NULL, (void *)pack);
+		package_tree = corba_packagedatastructlist_from_packagedata_tree (list);
+		g_list_free (list);
+
+		GNOME_Trilobite_Eazel_InstallCallback_uninstall_failed (service->callback, package_tree, &ev);
+		CORBA_free (package_tree);
 	} 
 	CORBA_exception_free (&ev);
 #endif /* EAZEL_INSTALL_NO_CORBA */
@@ -1713,10 +2115,14 @@ eazel_install_set_cgi_path (EazelInstall *service, const char *cgi_path)
 void eazel_install_set_debug (EazelInstall *service, gboolean debug) {
 	EAZEL_INSTALL_SANITY (service);
 	service->private->iopts->mode_debug = debug;
-
 	if (debug) {
+#ifdef ESKIL
 		eazel_package_system_set_debug (service->private->package_system, 
 						EAZEL_PACKAGE_SYSTEM_DEBUG_VERBOSE);
+#else
+		eazel_package_system_set_debug (service->private->package_system, 
+						EAZEL_PACKAGE_SYSTEM_DEBUG_FAIL);
+#endif
 	} else {
 		eazel_package_system_set_debug (service->private->package_system, 
 						EAZEL_PACKAGE_SYSTEM_DEBUG_FAIL);
@@ -1728,7 +2134,7 @@ ei_mutator_impl (silent, gboolean, iopts->mode_silent);
 ei_mutator_impl (test, gboolean, iopts->mode_test);
 ei_mutator_impl (force, gboolean, iopts->mode_force);
 ei_mutator_impl (depend, gboolean, iopts->mode_depend);
-ei_mutator_impl (update, gboolean, iopts->mode_update);
+ei_mutator_impl (upgrade, gboolean, iopts->mode_update);
 ei_mutator_impl (uninstall, gboolean, iopts->mode_uninstall);
 ei_mutator_impl (downgrade, gboolean, iopts->mode_downgrade);
 ei_mutator_impl (protocol, URLType, iopts->protocol);
@@ -1747,7 +2153,7 @@ ei_access_impl (debug, gboolean, iopts->mode_debug, FALSE);
 ei_access_impl (test, gboolean, iopts->mode_test, FALSE);
 ei_access_impl (force, gboolean, iopts->mode_force, FALSE);
 ei_access_impl (depend, gboolean, iopts->mode_depend, FALSE);
-ei_access_impl (update, gboolean, iopts->mode_update, FALSE);
+ei_access_impl (upgrade, gboolean, iopts->mode_update, FALSE);
 ei_access_impl (uninstall, gboolean, iopts->mode_uninstall, FALSE);
 ei_access_impl (downgrade, gboolean, iopts->mode_downgrade, FALSE);
 ei_access_impl (protocol, URLType , iopts->protocol, PROTOCOL_LOCAL);
