@@ -67,11 +67,19 @@ GdkPixbuf *eazel_services_pixbuf_new              (const char *name);
 GtkWidget *eazel_services_image_new               (const char *icon_name,
 						   const char *tile_name,
 						   guint32     background_color);
+GtkWidget *eazel_services_image_new_clickable     (const char *icon_name,
+						   const char *tile_name,
+						   guint32     background_color);
 GtkWidget *eazel_services_image_new_from_uri      (const char *uri,
 						   const char *tile_name,
 						   guint32     background_color,
 						   int         max_width,
 						   int         max_height);
+GtkWidget *eazel_services_clickable_image_new_from_uri (const char *uri,
+							const char *tile_name,
+							guint32     background_color,
+							int         max_width,
+							int         max_height);
 GtkWidget *eazel_services_label_new               (const char *text,
 						   guint       drop_shadow_offset,
 						   float       xalign,
@@ -83,6 +91,17 @@ GtkWidget *eazel_services_label_new               (const char *text,
 						   const char *tile_name,
 						   gint       num_larger_sizes,
 						   gboolean    bold);
+GtkWidget *eazel_services_label_new_clickable     (const char *text,
+						   guint       drop_shadow_offset,
+						   float       xalign,
+						   float       yalign,
+						   gint        xpadding,
+						   gint        ypadding,
+						   guint32     text_color,
+						   guint32     background_color,
+						   const char *tile_name,
+						   gint       num_larger_sizes,
+						   gboolean    bold);
 char *     eazel_services_get_current_date_string (void);
 
 END_GNOME_DECLS
