@@ -240,7 +240,7 @@ filtering_changed_callback (gpointer callback_data)
 	g_hash_table_foreach (directories, invalidate_one_count, NULL);
 }
 
-static void
+void
 emit_change_signals_for_all_files (NautilusDirectory *directory)
 {
 	GList *files;
@@ -709,6 +709,25 @@ nautilus_directory_find_file_by_relative_uri (NautilusDirectory *directory,
 	return node == NULL ? NULL : NAUTILUS_FILE (node->data);
 }
 
+NautilusFile *
+nautilus_directory_find_file_by_internal_uri (NautilusDirectory *directory,
+					      const char *relative_uri)
+{
+	NautilusFile *result;
+
+	if (nautilus_strcmp (relative_uri, ".") == 0) {
+		result = nautilus_directory_get_existing_corresponding_file (directory);
+		if (result != NULL) {
+			nautilus_file_unref (result);
+			g_return_val_if_fail (!GTK_OBJECT_DESTROYED (result), NULL);
+		}
+	} else {
+		result = nautilus_directory_find_file_by_relative_uri (directory, relative_uri);
+	}
+
+	return result;
+}
+
 void
 nautilus_directory_emit_files_added (NautilusDirectory *directory,
 				     GList *added_files)
