@@ -26,18 +26,23 @@
 #include "nautilus-global-preferences.h"
 
 #include "nautilus-file-utilities.h"
+#include "nautilus-font-manager.h"
 #include "nautilus-glib-extensions.h"
 #include "nautilus-gtk-extensions.h"
+#include "nautilus-medusa-support.h"
 #include "nautilus-preferences-dialog.h"
 #include "nautilus-preferences-group.h"
 #include "nautilus-preferences-item.h"
+#include "nautilus-scalable-font.h"
 #include "nautilus-string.h"
 #include "nautilus-medusa-support.h"
+#include "nautilus-stock-dialogs.h"
 #include "nautilus-view-identifier.h"
 #include <gconf/gconf.h>
 #include <gconf/gconf-client.h>
 #include <gtk/gtkbox.h>
 #include <libgnome/gnome-i18n.h>
+#include <libgnome/gnome-util.h>
 #include <liboaf/liboaf.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 
@@ -65,11 +70,13 @@ static void       global_preferences_install_speed_tradeoff_descriptions (const
 									  const char             *description);
 static void       global_preferences_install_home_location_defaults      (void);
 static void       global_preferences_install_medusa_defaults             (void);
+static void       global_preferences_install_font_defaults               (void);
 static void       global_preferences_install_descriptions                (void);
 static int        compare_view_identifiers                               (gconstpointer           a,
 									  gconstpointer           b);
 static GtkWidget *global_preferences_create_dialog                       (void);
 static GtkWidget *global_preferences_create_search_pane                  (NautilusPreferencesBox *preference_box);
+static GtkWidget *global_preferences_create_font_group                   (NautilusPreferencesPane *appearance_pane);
 
 static GtkWidget *global_prefs_dialog = NULL;
 
@@ -153,6 +160,8 @@ global_preferences_install_descriptions (void)
 	nautilus_preferences_set_description (NAUTILUS_PREFERENCES_DIRECTORY_VIEW_FONT_FAMILY,
 					      _("Use this font family to display file names:"));
 
+	nautilus_preferences_set_description (NAUTILUS_PREFERENCES_DIRECTORY_VIEW_SMOOTH_FONT,
+					      _("Use this font family to display file names:"));
 	
 	nautilus_preferences_set_description (NAUTILUS_PREFERENCES_START_WITH_TOOLBAR,
 					      _("Display toolbar in new windows"));
@@ -262,10 +271,6 @@ global_preferences_install_defaults (void)
 						  NAUTILUS_USER_LEVEL_INTERMEDIATE,
 						  NAUTILUS_SPEED_TRADEOFF_ALWAYS);
 	
-	nautilus_preferences_default_set_string (NAUTILUS_PREFERENCES_DIRECTORY_VIEW_FONT_FAMILY,
-						 NAUTILUS_USER_LEVEL_NOVICE,
-						 "helvetica");
-
 	nautilus_preferences_default_set_integer (NAUTILUS_PREFERENCES_CLICK_POLICY,
 						  NAUTILUS_USER_LEVEL_NOVICE,
 						  NAUTILUS_CLICK_POLICY_DOUBLE);
@@ -288,7 +293,7 @@ global_preferences_install_defaults (void)
 	
 	nautilus_preferences_default_set_boolean (NAUTILUS_PREFERENCES_SMOOTH_GRAPHICS_MODE,
 						  NAUTILUS_USER_LEVEL_NOVICE,
-						  TRUE);
+						  !nautilus_dumb_down_for_multi_byte_locale_hack ());
 	
 	nautilus_preferences_default_set_integer (NAUTILUS_PREFERENCES_PREVIEW_SOUND,
 						  NAUTILUS_USER_LEVEL_NOVICE,
@@ -374,6 +379,9 @@ global_preferences_install_defaults (void)
 	/* Home location */
 	global_preferences_install_home_location_defaults ();
 
+	/* Fonts */
+	global_preferences_install_font_defaults ();
+
 	/* Medusa */
 	global_preferences_install_medusa_defaults ();
 }
@@ -522,8 +530,7 @@ global_preferences_create_dialog (void)
 	 * Appearance
 	 */
 	appearance_pane = nautilus_preferences_box_add_pane (preference_box,
-							     _("Appearance"),
-							     _("Appearance Settings"));
+							     _("Appearance"));
 	
 	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (appearance_pane), _("Smoother Graphics"));
 	
@@ -531,20 +538,18 @@ global_preferences_create_dialog (void)
 							 0,
 							 NAUTILUS_PREFERENCES_SMOOTH_GRAPHICS_MODE,
 							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
-	
-	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (appearance_pane), _("Fonts"));
-	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (appearance_pane),
-							 1,
-							 NAUTILUS_PREFERENCES_DIRECTORY_VIEW_FONT_FAMILY,
-							 NAUTILUS_PREFERENCE_ITEM_FONT_FAMILY);
+
+	/*
+	 * Fonts
+	 */
+	global_preferences_create_font_group (NAUTILUS_PREFERENCES_PANE (appearance_pane));
 
 
 	/*
 	 * Windows & Desktop pane
 	 */
 	windows_and_desktop_pane = nautilus_preferences_box_add_pane (preference_box,
-								      _("Windows & Desktop"),
-								      _("Windows & Desktop Settings"));
+								      _("Windows & Desktop"));
 
 	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (windows_and_desktop_pane), _("Desktop"));
 	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (windows_and_desktop_pane),
@@ -587,8 +592,7 @@ global_preferences_create_dialog (void)
 	 * Folder Views pane
 	 */
 	directory_views_pane = nautilus_preferences_box_add_pane (preference_box,
-								 _("Icon & List Views"),
-								 _("Icon & List Views Settings"));
+								 _("Icon & List Views"));
 	
 	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (directory_views_pane), _("Click Behavior"));
 	
@@ -625,8 +629,7 @@ global_preferences_create_dialog (void)
 	 * Sidebar panels pane
 	 */
 	sidebar_panels_pane = nautilus_preferences_box_add_pane (preference_box,
-								 _("Sidebar Panels"),
-								 _("Sidebar Panels Settings"));
+								 _("Sidebar Panels"));
 	
 	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (sidebar_panels_pane), 
 					     _("Tabs"));
@@ -678,8 +681,7 @@ global_preferences_create_dialog (void)
 	 * Navigation
 	 */
 	navigation_pane = nautilus_preferences_box_add_pane (preference_box,
-							    _("Navigation"),
-							    _("Navigation Settings"));
+							    _("Navigation"));
 
 	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (navigation_pane), _("Home"));
 	
@@ -718,8 +720,7 @@ global_preferences_create_dialog (void)
 	 * Tradeoffs
 	 */
 	tradeoffs_pane = nautilus_preferences_box_add_pane (preference_box,
-							    _("Speed Tradeoffs"),
-							    _("Speed Tradeoffs Settings"));
+							    _("Speed Tradeoffs"));
 
 	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (tradeoffs_pane), _("Show Text in Icons"));
 	
@@ -790,8 +791,7 @@ global_preferences_create_search_pane (NautilusPreferencesBox *preference_box)
  	 * Search Settings 
  	 */
 	search_pane = nautilus_preferences_box_add_pane (preference_box,
-								_("Search"),
-								_("Search Settings"));
+							 _("Search"));
 	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (search_pane),
 					     _("Search Complexity Options"));
 	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (search_pane),
@@ -823,6 +823,57 @@ global_preferences_create_search_pane (NautilusPreferencesBox *preference_box)
 	return search_pane;
 }
 
+/*
+ * We have 2 font picker items, but we only show one depending on 
+ * the value of the SMOOTH_GRAPHICS preference.
+ */
+static void
+appearance_pane_update (gpointer callback_data)
+{
+	g_return_if_fail (NAUTILUS_IS_PREFERENCES_PANE (callback_data));
+
+	nautilus_preferences_pane_update (NAUTILUS_PREFERENCES_PANE (callback_data));
+}
+
+static GtkWidget *
+global_preferences_create_font_group (NautilusPreferencesPane *appearance_pane)
+{
+	GtkWidget *font_group;
+	GtkWidget *font_picker_smooth_item;
+	GtkWidget *font_picker_gdk_item;
+
+	font_group =
+		nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (appearance_pane), _("Fonts"));
+
+	font_picker_gdk_item = 
+		nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (appearance_pane),
+								 1,
+								 NAUTILUS_PREFERENCES_DIRECTORY_VIEW_FONT_FAMILY,
+								 NAUTILUS_PREFERENCE_ITEM_FONT_FAMILY);
+
+	nautilus_preferences_item_set_control_preference (NAUTILUS_PREFERENCES_ITEM (font_picker_gdk_item),
+							  NAUTILUS_PREFERENCES_SMOOTH_GRAPHICS_MODE);
+	nautilus_preferences_item_set_control_action (NAUTILUS_PREFERENCES_ITEM (font_picker_gdk_item),
+						      NAUTILUS_PREFERENCE_ITEM_HIDE);
+
+	font_picker_smooth_item =
+		nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (appearance_pane),
+								 1,
+								 NAUTILUS_PREFERENCES_DIRECTORY_VIEW_SMOOTH_FONT,
+								 NAUTILUS_PREFERENCE_ITEM_SMOOTH_FONT);
+	
+	nautilus_preferences_item_set_control_preference (NAUTILUS_PREFERENCES_ITEM (font_picker_smooth_item),
+							  NAUTILUS_PREFERENCES_SMOOTH_GRAPHICS_MODE);
+	nautilus_preferences_item_set_control_action (NAUTILUS_PREFERENCES_ITEM (font_picker_smooth_item),
+						      NAUTILUS_PREFERENCE_ITEM_SHOW);
+
+ 	nautilus_preferences_add_callback (NAUTILUS_PREFERENCES_SMOOTH_GRAPHICS_MODE,
+ 					   appearance_pane_update,
+ 					   appearance_pane);
+	
+	return font_group;
+}
+
 /* Make a query to find out what sidebar panels are available. */
 static GList *
 global_preferences_get_sidebar_panel_view_identifiers (void)
@@ -1057,24 +1108,89 @@ global_preferences_install_home_location_defaults (void)
 	g_free (default_intermediate_home_uri);
 }
 
+static void
+global_preferences_install_font_defaults (void)
+{
+	char *default_smooth_font;
+
+	nautilus_preferences_default_set_string (NAUTILUS_PREFERENCES_DIRECTORY_VIEW_FONT_FAMILY,
+						 NAUTILUS_USER_LEVEL_NOVICE,
+						 _("helvetica"));
+
+	/* The default smooth font */
+	default_smooth_font = nautilus_font_manager_get_default_font ();
+	
+	nautilus_preferences_default_set_string (NAUTILUS_PREFERENCES_DIRECTORY_VIEW_SMOOTH_FONT,
+						 NAUTILUS_USER_LEVEL_NOVICE,
+						 default_smooth_font);
+	g_free (default_smooth_font);
+}
+
 static void
 global_preferences_use_fast_search_changed_callback (gpointer callback_data)
 {
 	gboolean use_fast_search;
+	gboolean services_are_blocked;
+	NautilusCronStatus cron_status;
 
-	use_fast_search = nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_USE_FAST_SEARCH);
-
-	nautilus_medusa_enable_services (use_fast_search);
+	if (global_prefs_dialog) {
+		use_fast_search = nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_USE_FAST_SEARCH);
+		
+		nautilus_medusa_enable_services (use_fast_search);
+		services_are_blocked = nautilus_medusa_blocked ();
+		if (use_fast_search && !services_are_blocked) {
+			cron_status = nautilus_medusa_check_cron_is_enabled ();
+			switch (cron_status) {
+			case NAUTILUS_CRON_STATUS_OFF:
+				nautilus_show_info_dialog_with_details (_("Indexing is turned on, enabling the "
+									  "fast search feature. However, indexing "
+									  "currently can't be performed because "
+									  "the program crond, which does "
+									  "nightly tasks on your computer, "
+									  "is turned off. To make sure fast "
+									  "searches can be done, turn crond on."),
+									_("Files May Not Be Indexed"),
+									_("If you are running Linux, you can log "
+									  "in as root and type these commands "
+									  "to start cron:\n\n"
+									  "/sbin/chkconfig --level 345 crond on\n"
+									  "/etc/rc.d/init.d/cron start\n"),
+									NULL);
+				
+				break;
+			case NAUTILUS_CRON_STATUS_UNKNOWN:
+				nautilus_show_info_dialog_with_details (_("Indexing is turned on, enabling the "
+									  "fast search feature. However, indexing "
+									  "may not be performed because the "
+									  "program crond, which does nightly "
+									  "tasks on your computer, may be turned "
+									  "off. To make sure fast searches can be "
+									  "done, check to make sure that crond "
+									  "is turned on.\n\n"),
+									_("Files May Not Be Indexed"),
+									_("If you are running Linux, you can log "
+									  "in as root and type these commands "
+									  "to start cron:\n\n"
+									  "/sbin/chkconfig --level 345 crond on\n"
+									  "/etc/rc.d/init.d/cron start\n"),
+									NULL);
+				break;
+			default:
+				break;
+			}
+		}
+	}
 }
-
+	
 static void 
 global_preferences_medusa_state_changed_callback (gpointer callback_data)
 {
 	nautilus_preferences_set_boolean (NAUTILUS_PREFERENCES_USE_FAST_SEARCH,
 					  nautilus_medusa_services_have_been_enabled_by_user ());
-
+	
 	nautilus_preferences_set_boolean (NAUTILUS_PREFERENCES_MEDUSA_BLOCKED,
 					  nautilus_medusa_blocked ());
+
 }
 
 static void
@@ -1145,6 +1261,60 @@ nautilus_global_preferences_set_dialog_title (const char *title)
 	gtk_window_set_title (GTK_WINDOW (dialog), title);
 }
 
+/**
+ * nautilus_global_preferences_get_smooth_font
+ *
+ * Return value: The user's preferred smooth font.  Need to 
+ *               unref the returned GtkObject when done with it.
+ */
+NautilusScalableFont *
+nautilus_global_preferences_get_smooth_font (void)
+{
+	NautilusScalableFont *scalable_font;
+	char *font_file_name;
+
+	font_file_name = nautilus_preferences_get (NAUTILUS_PREFERENCES_DIRECTORY_VIEW_SMOOTH_FONT);
+	
+	scalable_font = 
+		(font_file_name && g_file_exists (font_file_name)) ?
+		nautilus_scalable_font_new (font_file_name) :
+		nautilus_scalable_font_get_default_font ();
+	g_free (font_file_name);
+	
+	g_assert (NAUTILUS_IS_SCALABLE_FONT (scalable_font));
+
+	return scalable_font;
+}
+
+/**
+ * nautilus_global_preferences_get_smooth_bold_font
+ *
+ * Return value: A bold flavor on the user's preferred smooth font.  If
+ *               no bold font is found, then the plain preffered font is
+ *               used. Need to unref the returned GtkObject when done
+ *               with it.
+ */
+NautilusScalableFont *
+nautilus_global_preferences_get_smooth_bold_font (void)
+{
+	NautilusScalableFont *plain_font;
+	NautilusScalableFont *bold_font;
+
+	plain_font = nautilus_global_preferences_get_smooth_font ();
+	g_assert (NAUTILUS_IS_SCALABLE_FONT (plain_font));
+
+	bold_font = nautilus_scalable_font_make_bold (plain_font);
+
+	if (bold_font == NULL) {
+		bold_font = plain_font;
+	} else {
+		gtk_object_unref (GTK_OBJECT (plain_font));
+	}
+
+	g_assert (NAUTILUS_IS_SCALABLE_FONT (bold_font));
+	return bold_font;
+}
+
 void
 nautilus_global_preferences_initialize (void)
 {
