@@ -52,6 +52,7 @@
 #include "nautilus-icon-factory.h"
 #include "nautilus-theme.h"
 #include "nautilus-smooth-text-layout.h"
+#include "nautilus-smooth-text-layout-cache.h"
 
 /* Comment this out if the new smooth fonts code give you problems
  * This isnt meant to be permanent.  Its just a precaution.
@@ -60,6 +61,7 @@
 
 /* gap between bottom of icon and start of text box */
 #define LABEL_OFFSET 1
+#define LABEL_LINE_SPACING 0
 
 #define MAX_TEXT_WIDTH_STANDARD 135
 #define MAX_TEXT_WIDTH_TIGHTER 80
@@ -221,6 +223,14 @@ static gboolean icon_canvas_item_is_smooth                 (const NautilusIconCa
 
 NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusIconCanvasItem, nautilus_icon_canvas_item, GNOME_TYPE_CANVAS_ITEM)
 
+static NautilusSmoothTextLayoutCache *layout_cache;
+
+static void
+free_layout_cache (void)
+{
+	gtk_object_unref (GTK_OBJECT (layout_cache));
+}
+
 /* Class initialization function for the icon canvas item. */
 static void
 nautilus_icon_canvas_item_initialize_class (NautilusIconCanvasItemClass *class)
@@ -228,6 +238,11 @@ nautilus_icon_canvas_item_initialize_class (NautilusIconCanvasItemClass *class)
 	GtkObjectClass *object_class;
 	GnomeCanvasItemClass *item_class;
 
+	if (layout_cache == NULL) {
+		layout_cache = nautilus_smooth_text_layout_cache_new ();
+		g_atexit (free_layout_cache);
+	}
+
 	object_class = GTK_OBJECT_CLASS (class);
 	item_class = GNOME_CANVAS_ITEM_CLASS (class);
 
@@ -1356,7 +1371,7 @@ draw_or_measure_label_text_aa (NautilusIconCanvasItem *item,
 	GnomeCanvasItem *canvas_item;
 	int max_text_width;
 	int icon_width, text_left, box_left;
-	NautilusSmoothTextLayout *smooth_text_layout;
+	const NautilusSmoothTextLayout *smooth_text_layout;
 	char **pieces;
 	const char *text_piece;
 	int i;
@@ -1448,14 +1463,13 @@ draw_or_measure_label_text_aa (NautilusIconCanvasItem *item,
 			text_piece = " ";
 		}
 
-		smooth_text_layout = nautilus_smooth_text_layout_new (text_piece,
-								      strlen (text_piece),
-								      details->smooth_font,
-								      details->smooth_font_size,
-								      TRUE);
-		nautilus_smooth_text_layout_set_line_spacing (smooth_text_layout, 2);
-		nautilus_smooth_text_layout_set_empty_line_height (smooth_text_layout, 20);
-		nautilus_smooth_text_layout_set_line_wrap_width (smooth_text_layout, max_text_width);
+		smooth_text_layout = nautilus_smooth_text_layout_cache_render (layout_cache,
+									       text_piece,
+									       strlen (text_piece),
+									       details->smooth_font,
+									       details->smooth_font_size,
+									       TRUE, LABEL_LINE_SPACING,
+									       max_text_width);
 		
 		/* Draw text if we are not in user rename mode */
 		if (destination_pixbuf != NULL && !details->is_renaming) {
@@ -1521,7 +1535,7 @@ draw_or_measure_label_text_aa (NautilusIconCanvasItem *item,
 		}
 		
 		width_so_far = MAX (width_so_far, (guint) nautilus_smooth_text_layout_get_width (smooth_text_layout));
-		height_so_far += nautilus_smooth_text_layout_get_height (smooth_text_layout);
+		height_so_far += nautilus_smooth_text_layout_get_height (smooth_text_layout) + LABEL_LINE_SPACING;
 		
 		gtk_object_unref (GTK_OBJECT (smooth_text_layout));
 	}
