@@ -2,7 +2,7 @@
 
 /* nautilus-mime-actions.h - uri-specific versions of mime action functions
 
-   Copyright (C) 2000 Eazel, Inc.
+   Copyright (C) 2000, 2001 Eazel, Inc.
 
    The Gnome Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public License as
@@ -30,6 +30,7 @@
 #include "nautilus-glib-extensions.h"
 #include "nautilus-lib-self-check-functions.h"
 #include "nautilus-metadata.h"
+#include "nautilus-string.h"
 #include <libgnomevfs/gnome-vfs-application-registry.h>
 #include <libgnomevfs/gnome-vfs-mime-info.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
@@ -672,15 +673,47 @@ nautilus_mime_get_all_applications_for_file (NautilusFile      *file)
 	return result;
 }
 
+static int
+application_supports_uri_scheme_strcmp_style (gconstpointer application_data,
+					      gconstpointer uri_scheme)
+{
+	return application_supports_uri_scheme
+		((gpointer) application_data,
+		 (gpointer) uri_scheme) ? 0 : 1;
+}
+
 gboolean
 nautilus_mime_has_any_applications_for_file (NautilusFile      *file)
 {
-	GList *list;
+	GList *all_applications_for_mime_type, *application_that_can_access_uri;
+	char *uri_scheme;
 	gboolean result;
 
-	list = nautilus_mime_get_all_applications_for_file (file);
-	result = list != NULL;
-	gnome_vfs_mime_application_list_free (list);
+	all_applications_for_mime_type = nautilus_mime_get_all_applications_for_file (file);
+
+	uri_scheme = nautilus_file_get_uri_scheme (file);
+	application_that_can_access_uri = g_list_find_custom
+		(all_applications_for_mime_type,
+		 uri_scheme,
+		 application_supports_uri_scheme_strcmp_style);
+	g_free (uri_scheme);
+
+	result = application_that_can_access_uri != NULL;
+	gnome_vfs_mime_application_list_free (all_applications_for_mime_type);
+
+	return result;
+}
+
+gboolean
+nautilus_mime_has_any_applications_for_file_type (NautilusFile      *file)
+{
+	GList *applications;
+	gboolean result;
+
+	applications = nautilus_mime_get_all_applications_for_file (file);
+	
+	result = applications != NULL;
+	gnome_vfs_mime_application_list_free (applications);
 
 	return result;
 }
@@ -1638,13 +1671,11 @@ application_supports_uri_scheme (gpointer data,
 	application = (GnomeVFSMimeApplication *) data;
 
 	/* The default supported uri scheme is "file" */
-	if (application->supported_uri_schemes == NULL &&
-	    strcmp ((const char *)uri_scheme, "file") == 0) {
+	if (application->supported_uri_schemes == NULL
+	    && strcasecmp ((const char *) uri_scheme, "file") == 0) {
 		return TRUE;
 	}
-	return (g_list_find_custom (application->supported_uri_schemes,
-				    uri_scheme,
-				    (GCompareFunc) strcmp) != NULL);
-
-	
+	return g_list_find_custom (application->supported_uri_schemes,
+				   uri_scheme,
+				   nautilus_strcasecmp_compare_func) != NULL;
 }
