@@ -35,7 +35,7 @@
 #include <libgnome/gnome-i18n.h>
 
 #define MIN_FONT_SIZE 5
-#define DEFAULT_LINE_SPACING 2
+#define DEFAULT_LINE_SPACING 0
 #define DEFAULT_FONT_SIZE 14
 
 #define UNDEFINED_DIMENSION -1
@@ -181,6 +181,10 @@ nautilus_smooth_text_layout_destroy (GtkObject *object)
 	smooth_text_layout = NAUTILUS_SMOOTH_TEXT_LAYOUT (object);
 
 	smooth_text_layout_clear_lines (smooth_text_layout);
+	/* FIXME: it seems like we should unref the font here. */
+	g_free (smooth_text_layout->details->line_break_characters);
+	g_free (smooth_text_layout->details->text);
+
 	g_free (smooth_text_layout->details);
 
 	/* Chain destroy */
@@ -203,6 +207,7 @@ smooth_text_layout_clear_lines (NautilusSmoothTextLayout *smooth_text_layout)
 	smooth_text_layout->details->empty_line_height = UNDEFINED_DIMENSION;
 	smooth_text_layout->details->line_wrap_width = UNDEFINED_DIMENSION;
 	smooth_text_layout->details->total_line_height = UNDEFINED_DIMENSION;
+
 }
 
 static void
@@ -363,15 +368,11 @@ smooth_text_layout_line_list_draw_to_pixbuf (GList *text_line_list,
 			if (underlined) {
 				ArtIRect underline_rect;
 
-				/* FIXME bugzilla.eazel.com 2865: This underlining code should
-				 * take into account the baseline.
-				 */
-				const int underline_height = 1;
-
-				underline_rect = nautilus_glyph_intersect (glyph, text_x, text_y, NULL);
-				underline_rect = nautilus_art_irect_inset (underline_rect, 1, 0);
-  				underline_rect.y0 = underline_rect.y1 - underline_height;
-				
+				nautilus_glyph_get_underline_rectangle (glyph, &underline_rect);
+				underline_rect.y0 += text_y;
+				underline_rect.y1 += text_y;
+				underline_rect.x0 += text_x;
+				underline_rect.x1 += text_x;
 				nautilus_gdk_pixbuf_fill_rectangle_with_color (pixbuf,
 									       &underline_rect,
 									       color);
@@ -642,11 +643,7 @@ nautilus_smooth_text_layout_draw_to_pixbuf (const NautilusSmoothTextLayout *smoo
 	g_return_if_fail (nautilus_gdk_pixbuf_is_valid (destination_pixbuf));
 	g_return_if_fail (destination_area != NULL);
 	g_return_if_fail (justification >= GTK_JUSTIFY_LEFT && justification <= GTK_JUSTIFY_FILL);
-
-	/* Make sure there is some work to do */
-	if (art_irect_empty (destination_area)) {
-		return;
-	}
+	g_return_if_fail (!art_irect_empty (destination_area));
 
 	smooth_text_layout_ensure_lines (smooth_text_layout);
 	
@@ -698,6 +695,101 @@ nautilus_smooth_text_layout_draw_to_pixbuf (const NautilusSmoothTextLayout *smoo
 	gdk_pixbuf_unref (sub_pixbuf);
 }
 
+void
+nautilus_smooth_text_layout_draw_to_pixbuf_shadow (const NautilusSmoothTextLayout *smooth_text_layout,
+						   GdkPixbuf *destination_pixbuf,
+						   int source_x,
+						   int source_y,
+						   const ArtIRect *destination_area,
+						   int shadow_offset,
+						   GtkJustification justification,
+						   gboolean underlined,
+						   guint32 color,
+						   guint32 shadow_color,
+						   int opacity)
+{
+	NautilusDimensions dimensions;
+	ArtIRect target;
+	ArtIRect source;
+	int target_width;
+	int target_height;
+	int source_width;
+	int source_height;
+	GdkPixbuf *sub_pixbuf;
+
+	g_return_if_fail (NAUTILUS_IS_SMOOTH_TEXT_LAYOUT (smooth_text_layout));
+	g_return_if_fail (nautilus_gdk_pixbuf_is_valid (destination_pixbuf));
+	g_return_if_fail (destination_area != NULL);
+	g_return_if_fail (shadow_offset > 0);
+	g_return_if_fail (justification >= GTK_JUSTIFY_LEFT && justification <= GTK_JUSTIFY_FILL);
+	g_return_if_fail (!art_irect_empty (destination_area));
+
+	smooth_text_layout_ensure_lines (smooth_text_layout);
+	
+	dimensions = nautilus_smooth_text_layout_get_dimensions (smooth_text_layout);
+	dimensions.width += shadow_offset;
+	dimensions.height += shadow_offset;
+
+	g_return_if_fail (source_x >= 0);
+	g_return_if_fail (source_y >= 0);
+	g_return_if_fail (source_x < dimensions.width);
+	g_return_if_fail (source_y < dimensions.height);
+
+	/* Clip the destination area to the pixbuf dimensions; bail if no work */
+	target = nautilus_gdk_pixbuf_intersect (destination_pixbuf, 0, 0, destination_area);
+	if (art_irect_empty (&target)) {
+ 		return;
+ 	}
+
+	/* Assign the source area */
+	nautilus_art_irect_assign (&source,
+				   source_x,
+				   source_y,
+				   dimensions.width - source_x,
+				   dimensions.height - source_y);
+
+	/* Adjust the target width if the source area is smaller than the
+	 * source pixbuf dimensions */
+	target_width = target.x1 - target.x0;
+	target_height = target.y1 - target.y0;
+	source_width = source.x1 - source.x0;
+	source_height = source.y1 - source.y0;
+
+	target.x1 = target.x0 + MIN (target_width, source_width);
+	target.y1 = target.y0 + MIN (target_height, source_height);
+
+	/* Use a sub area pixbuf for simplicity */
+	sub_pixbuf = nautilus_gdk_pixbuf_new_from_pixbuf_sub_area (destination_pixbuf, &target);
+
+	/* Draw the shadow text */
+	smooth_text_layout_line_list_draw_to_pixbuf (smooth_text_layout->details->text_line_list,
+						     sub_pixbuf,
+						     -source_x + shadow_offset,
+						     -source_y + shadow_offset,
+						     justification,
+						     underlined,
+						     smooth_text_layout_get_empty_line_height (smooth_text_layout),
+						     smooth_text_layout_get_max_line_width (smooth_text_layout),
+						     smooth_text_layout->details->line_spacing,
+						     shadow_color,
+						     opacity);
+
+	/* Draw the text */
+	smooth_text_layout_line_list_draw_to_pixbuf (smooth_text_layout->details->text_line_list,
+						     sub_pixbuf,
+						     -source_x,
+						     -source_y,
+						     justification,
+						     underlined,
+						     smooth_text_layout_get_empty_line_height (smooth_text_layout),
+						     smooth_text_layout_get_max_line_width (smooth_text_layout),
+						     smooth_text_layout->details->line_spacing,
+						     color,
+						     opacity);
+
+	gdk_pixbuf_unref (sub_pixbuf);
+}
+
 NautilusDimensions
 nautilus_smooth_text_layout_get_dimensions (const NautilusSmoothTextLayout *smooth_text_layout)
 {
@@ -1143,3 +1235,55 @@ nautilus_text_layout_new (const NautilusScalableFont *font,
 
 	return text_layout;
 }
+
+#if !defined (NAUTILUS_OMIT_SELF_CHECK)
+
+gboolean
+nautilus_smooth_text_layout_compare (NautilusSmoothTextLayout *x,
+				     NautilusSmoothTextLayout *y)
+{
+	GList *xp, *yp;
+
+	g_return_val_if_fail (NAUTILUS_IS_SMOOTH_TEXT_LAYOUT (x), FALSE);
+	g_return_val_if_fail (NAUTILUS_IS_SMOOTH_TEXT_LAYOUT (y), FALSE);
+
+	/* Compare simple things */
+	if (x->details->dimensions.width != y->details->dimensions.width
+	    || x->details->dimensions.height != y->details->dimensions.height
+	    || x->details->text_length != y->details->text_length
+	    || memcmp (x->details->text, y->details->text, x->details->text_length) != 0
+	    || x->details->font != y->details->font
+	    || x->details->font_size != y->details->font_size
+	    || x->details->line_spacing != y->details->line_spacing
+	    || x->details->empty_line_height != y->details->empty_line_height
+	    || x->details->max_line_width != y->details->max_line_width
+	    || x->details->num_empty_lines != y->details->num_empty_lines
+	    || x->details->line_wrap_width != y->details->line_wrap_width
+	    || x->details->total_line_height != y->details->total_line_height
+	    || x->details->wrap != y->details->wrap
+	    || strcmp (x->details->line_break_characters, y->details->line_break_characters) != 0) {
+		return FALSE;
+	}
+
+	/* Compare glyphs */
+	smooth_text_layout_ensure_lines (x);
+	smooth_text_layout_ensure_lines (y);
+	for (xp = x->details->text_line_list, yp = y->details->text_line_list;
+	     xp != NULL && yp != NULL;
+	     xp = xp->next, yp = yp->next) {
+		if ((xp->data == NULL) != (yp->data == NULL)) {
+			return FALSE;
+		}
+		if (xp->data != NULL
+		    && !nautilus_glyph_compare (xp->data, yp->data)) {
+			return FALSE;
+		}
+	}
+	if (xp != NULL || yp != NULL) {
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+#endif /* NAUTILUS_OMIT_SELF_CHECK */
