@@ -162,6 +162,8 @@ static void     	get_iso9660_volume_name                         (NautilusVolume
 static void     	get_ext2_volume_name                         	(NautilusVolume             	*volume);
 static void     	get_msdos_volume_name                         	(NautilusVolume             	*volume);
 static void     	get_nfs_volume_name                         	(NautilusVolume             	*volume);
+static void     	get_reiser_volume_name                         	(NautilusVolume             	*volume);
+static void             get_ufs_volume_name                             (NautilusVolume                *volume);
 static void     	get_floppy_volume_name                         	(NautilusVolume             	*volume);
 static void     	get_generic_volume_name                         (NautilusVolume             	*volume);
 static void		mount_volume_get_name 				(NautilusVolume 		*volume);
@@ -554,7 +556,7 @@ modify_volume_name_for_display (NautilusVolume *volume)
 	
 	/* Save pretty name back into volume info */
 	g_free (volume->volume_name);
-	volume->volume_name = g_strdup (name);
+	volume->volume_name = name;
 }
 
 /* nautilus_volume_monitor_get_target_uri
@@ -593,6 +595,8 @@ nautilus_volume_monitor_should_integrate_trash (const NautilusVolume *volume)
 		|| volume->type == NAUTILUS_VOLUME_NFS
 		|| volume->type == NAUTILUS_VOLUME_VFAT
 		|| volume->type == NAUTILUS_VOLUME_FLOPPY
+		|| volume->type == NAUTILUS_VOLUME_REISER
+		|| volume->type == NAUTILUS_VOLUME_UFS
 		|| volume->type == NAUTILUS_VOLUME_SMB;
 }
 
@@ -782,12 +786,18 @@ mount_volume_get_name (NautilusVolume *volume)
 		get_nfs_volume_name (volume);
 		break;
 		
+	case NAUTILUS_VOLUME_REISER:
+		get_reiser_volume_name (volume);
+		break;
+	case NAUTILUS_VOLUME_UFS:
+		get_ufs_volume_name (volume);
+		break;
+
 	case NAUTILUS_VOLUME_AFFS:
 	case NAUTILUS_VOLUME_HPFS:
 	case NAUTILUS_VOLUME_MINIX:
 	case NAUTILUS_VOLUME_SMB:
 	case NAUTILUS_VOLUME_UDF:
-	case NAUTILUS_VOLUME_UFS:
 	case NAUTILUS_VOLUME_UNSDOS:
 	case NAUTILUS_VOLUME_XENIX:
 	case NAUTILUS_VOLUME_XIAFS:
@@ -833,9 +843,10 @@ mount_volume_activate (NautilusVolumeMonitor *monitor, NautilusVolume *volume)
 	case NAUTILUS_VOLUME_FAT:
 	case NAUTILUS_VOLUME_HPFS:
 	case NAUTILUS_VOLUME_MINIX:
+	case NAUTILUS_VOLUME_REISER:
+	case NAUTILUS_VOLUME_UFS:
 	case NAUTILUS_VOLUME_SMB:
 	case NAUTILUS_VOLUME_UDF:
-	case NAUTILUS_VOLUME_UFS:
 	case NAUTILUS_VOLUME_UNSDOS:
 	case NAUTILUS_VOLUME_VFAT:
 	case NAUTILUS_VOLUME_XENIX:
@@ -1320,6 +1331,19 @@ mount_volume_proc_add (NautilusVolume *volume)
 	volume->type = NAUTILUS_VOLUME_EXT2;
 	return TRUE;
 }
+static gboolean
+mount_volume_reiserfs_add (NautilusVolume *volume)
+{
+	volume->type = NAUTILUS_VOLUME_REISER;
+	return TRUE;
+}
+
+static gboolean
+mount_volume_ufs_add (NautilusVolume *volume)
+{
+        volume->type = NAUTILUS_VOLUME_UFS;
+        return TRUE;
+}
 
 static gboolean
 mount_volume_smb_add (NautilusVolume *volume)
@@ -1793,6 +1817,37 @@ get_nfs_volume_name (NautilusVolume *volume)
 	}
 }
 
+static void
+get_reiser_volume_name (NautilusVolume *volume)
+{
+	char *name;
+
+	name = strrchr (volume->mount_path, '/');
+	if (name != NULL) {
+		name++;
+		volume->volume_name = g_strdup (name);
+		modify_volume_name_for_display (volume);
+	} else {
+		volume->volume_name = g_strdup (_("ReiserFS Volume"));
+	}
+}
+
+
+static void
+get_ufs_volume_name (NautilusVolume *volume)
+{
+	char *name;
+
+	name = strrchr (volume->mount_path, '/');
+	if (name != NULL) {
+		name++;
+		volume->volume_name = g_strdup (name);
+		modify_volume_name_for_display (volume);
+	} else {
+		volume->volume_name = g_strdup (_("UFS Volume"));
+	}
+}
+
 static void
 get_floppy_volume_name (NautilusVolume *volume)
 {
@@ -1852,6 +1907,10 @@ mount_volume_add_filesystem (NautilusVolume *volume, GList *volume_list)
 		mounted = mount_volume_nfs_add (volume);
 	} else if (strcmp (volume->filesystem, "proc") == 0) {		
 		mounted = mount_volume_proc_add (volume);
+	} else if (strcmp (volume->filesystem, "reiserfs") == 0) {
+		mounted = mount_volume_reiserfs_add (volume);
+	} else if (strcmp (volume->filesystem, "ufs") == 0) {
+		mounted = mount_volume_ufs_add (volume);
 	} else if (strcmp (volume->filesystem, "smb") == 0) {		
 		mounted = mount_volume_smb_add (volume);
 	} else if (strcmp (volume->filesystem, "udf") == 0) {		
