@@ -30,8 +30,8 @@
 
 xmlNodePtr nautilus_xml_get_children                        (xmlNodePtr  parent);
 xmlNodePtr nautilus_xml_get_root_children                   (xmlDocPtr   document);
-xmlNodePtr nautilus_xml_get_child_by_name		    (xmlNodePtr  parent,
-							     const char  *child_name);
+xmlNodePtr nautilus_xml_get_child_by_name                   (xmlNodePtr  parent,
+							     const char *child_name);
 xmlNodePtr nautilus_xml_get_child_by_name_and_property      (xmlNodePtr  parent,
 							     const char *child_name,
 							     const char *property_name,
@@ -43,6 +43,8 @@ xmlNodePtr nautilus_xml_get_root_child_by_name_and_property (xmlDocPtr   documen
 GList     *nautilus_xml_get_property_for_children           (xmlNodePtr  parent,
 							     const char *child_name,
 							     const char *property_name);
-
+xmlChar   *nautilus_xml_get_property_translated             (xmlNodePtr  parent,
+							     const char *property_name);
+void       nautilus_xml_remove_node                         (xmlNodePtr  node);
 
 #endif /* NAUTILUS_XML_EXTENSIONS_H */
