@@ -1,26 +1,29 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 
+/*
+ * Nautilus
+ *
+ * Copyright (C) 1999, 2000 Eazel, Inc.
+ *
+ * Nautilus is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * Nautilus is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * Authors: John Sullivan <sullivan@eazel.com>
+ */
+
 /* nautilus-bookmark-list.c - implementation of centralized list of bookmarks.
-
-   Copyright (C) 1999, 2000 Eazel, Inc.
-
-   The Gnome Library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public License as
-   published by the Free Software Foundation; either version 2 of the
-   License, or (at your option) any later version.
-
-   The Gnome Library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public
-   License along with the Gnome Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
-
-   Authors: John Sullivan <sullivan@eazel.com>
-*/
+ */
 
 #include <config.h>
 #include "nautilus-bookmark-list.h"
@@ -38,25 +41,28 @@
 #include <libnautilus-extensions/nautilus-string.h>
 #include <libnautilus-extensions/nautilus-xml-extensions.h>
 
-#include <parser.h>
-#include <tree.h>
-#include <xmlmemory.h>
+#include <gnome-xml/parser.h>
+#include <gnome-xml/tree.h>
+#include <gnome-xml/xmlmemory.h>
 
 enum {
 	CONTENTS_CHANGED,
 	LAST_SIGNAL
 };
 
+static guint signals[LAST_SIGNAL];
+static char *window_geometry;
+
 /* forward declarations */
 static void        append_bookmark_node                 (gpointer              list_element,
 							 gpointer              user_data);
+static void        destroy                              (GtkObject            *object);
 static const char *nautilus_bookmark_list_get_file_path (NautilusBookmarkList *bookmarks);
 static void        nautilus_bookmark_list_load_file     (NautilusBookmarkList *bookmarks);
 static void        nautilus_bookmark_list_save_file     (NautilusBookmarkList *bookmarks);
 static void        set_window_geometry_internal         (const char           *string);
-
-static guint signals[LAST_SIGNAL];
-static char *window_geometry;
+static void        stop_monitoring_bookmark             (NautilusBookmarkList *bookmarks,
+							 NautilusBookmark     *bookmark);
 
 /* Initialization.  */
 
@@ -67,9 +73,11 @@ nautilus_bookmark_list_initialize_class (NautilusBookmarkListClass *class)
 
 	object_class = GTK_OBJECT_CLASS (class);
 
+	object_class->destroy = destroy;
+
 	signals[CONTENTS_CHANGED] =
 		gtk_signal_new ("contents_changed",
-				GTK_RUN_FIRST,
+				GTK_RUN_LAST,
 				object_class->type,
 				GTK_SIGNAL_OFFSET (NautilusBookmarkListClass, 
 						   contents_changed),
@@ -89,7 +97,30 @@ nautilus_bookmark_list_initialize (NautilusBookmarkList *bookmarks)
 
 NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusBookmarkList, nautilus_bookmark_list, GTK_TYPE_OBJECT)
 
-
+static void
+stop_monitoring_one (gpointer data, gpointer user_data)
+{
+	g_assert (NAUTILUS_IS_BOOKMARK (data));
+	g_assert (NAUTILUS_IS_BOOKMARK_LIST (user_data));
+
+	stop_monitoring_bookmark (NAUTILUS_BOOKMARK_LIST (user_data), 
+				  NAUTILUS_BOOKMARK (data));
+}		  
+
+static void
+clear (NautilusBookmarkList *bookmarks)
+{
+	g_list_foreach (bookmarks->list, stop_monitoring_one, bookmarks);
+	nautilus_gtk_object_list_free (bookmarks->list);
+	bookmarks->list = NULL;
+}
+
+static void
+destroy (GtkObject *object)
+{
+	clear (NAUTILUS_BOOKMARK_LIST (object));
+}
+
 /**
  * append_bookmark_node:
  * 
@@ -144,7 +175,6 @@ bookmark_in_list_changed_callback (NautilusBookmark *bookmark,
 	nautilus_bookmark_list_contents_changed (bookmarks);
 }				   
 				   
-
 static void
 stop_monitoring_bookmark (NautilusBookmarkList *bookmarks,
 			  NautilusBookmark *bookmark)
@@ -154,16 +184,6 @@ stop_monitoring_bookmark (NautilusBookmarkList *bookmarks,
 				       bookmarks);
 }
 
-static void
-stop_monitoring_one (gpointer data, gpointer user_data)
-{
-	g_assert (NAUTILUS_IS_BOOKMARK (data));
-	g_assert (NAUTILUS_IS_BOOKMARK_LIST (user_data));
-
-	stop_monitoring_bookmark (NAUTILUS_BOOKMARK_LIST (user_data), 
-				  NAUTILUS_BOOKMARK (data));
-}		  
-
 static void
 insert_bookmark_internal (NautilusBookmarkList *bookmarks,
 			  NautilusBookmark *bookmark,
@@ -262,7 +282,7 @@ nautilus_bookmark_list_delete_item_at (NautilusBookmarkList *bookmarks,
 	stop_monitoring_bookmark (bookmarks, NAUTILUS_BOOKMARK (doomed->data));
 	gtk_object_unref (GTK_OBJECT (doomed->data));
 	
-	g_list_free (doomed);
+	g_list_free_1 (doomed);
 	
 	nautilus_bookmark_list_contents_changed (bookmarks);
 }
@@ -294,7 +314,7 @@ nautilus_bookmark_list_delete_items_with_uri (NautilusBookmarkList *bookmarks,
 			bookmarks->list = g_list_remove_link (bookmarks->list, node);
 			stop_monitoring_bookmark (bookmarks, NAUTILUS_BOOKMARK (node->data));
 			gtk_object_unref (GTK_OBJECT (node->data));
-			g_list_free (node);
+			g_list_free_1 (node);
 			list_changed = TRUE;
 		}
 		g_free (bookmark_uri);
@@ -410,10 +430,7 @@ nautilus_bookmark_list_load_file (NautilusBookmarkList *bookmarks)
 	xmlNodePtr node;
 
 	/* Wipe out old list. */
-	g_list_foreach (bookmarks->list, stop_monitoring_one, bookmarks);
-	nautilus_gtk_object_list_free (bookmarks->list);
-	bookmarks->list = NULL;
-	
+	clear (bookmarks);
 
 	/* Read new list from file */
 	doc = xmlParseFile (nautilus_bookmark_list_get_file_path (bookmarks));
@@ -447,7 +464,13 @@ nautilus_bookmark_list_load_file (NautilusBookmarkList *bookmarks)
 NautilusBookmarkList *
 nautilus_bookmark_list_new (void)
 {
-	return gtk_type_new (NAUTILUS_TYPE_BOOKMARK_LIST);
+	NautilusBookmarkList *list;
+
+	list = NAUTILUS_BOOKMARK_LIST (gtk_object_new (NAUTILUS_TYPE_BOOKMARK_LIST, NULL));
+	gtk_object_ref (GTK_OBJECT (list));
+	gtk_object_sink (GTK_OBJECT (list));
+
+	return list;
 }
 
 /**
