@@ -42,6 +42,7 @@
 #include <libnautilus-extensions/nautilus-image.h>
 #include <libnautilus-extensions/nautilus-label.h>
 #include <libnautilus-extensions/nautilus-link.h>
+#include <libnautilus-extensions/nautilus-medusa-support.h>
 #include <libnautilus-extensions/nautilus-preferences.h>
 #include <libnautilus-extensions/nautilus-radio-button-group.h>
 #include <libnautilus-extensions/nautilus-string.h>
@@ -62,11 +63,12 @@
 #define SERVICE_UPDATE_ARCHIVE_PATH "/tmp/nautilus_update.tgz"
 #define WELCOME_PACKAGE_URI "http://services.eazel.com/downloads/eazel/updates.tgz"
 
-/* Wizard page number enumeration */
+/* Druid page number enumeration */
 enum {
 	USER_LEVEL_PAGE = 0,
 	GMC_TRANSITION_PAGE,
 	LAUNCH_MEDUSA_PAGE,
+	START_CRON_INFORMATION_PAGE,
 	OFFER_UPDATE_PAGE,
 	UPDATE_FEEDBACK_PAGE,
 	PROXY_CONFIGURATION_PAGE,
@@ -105,6 +107,7 @@ static GtkWidget *finish_page;
 static GtkWidget *pages[NUMBER_OF_STANDARD_PAGES];
 
 static GtkWidget *download_label;
+static GtkWidget *finished_label;
 
 static int last_update_choice = 0;
 static int last_proxy_choice = 1;
@@ -112,6 +115,11 @@ static int last_proxy_choice = 1;
 static GtkWidget *port_number_entry;
 static GtkWidget *proxy_address_entry;
 
+/* Only set true when we're absolutely totally positive that we've got
+ * an http connection to the outside world.
+ */
+static gboolean http_is_known_to_work = FALSE;
+
 /* Set by set_http_proxy; used by check_network_connectivity */
 
 /* NULL indicates no HTTP proxy */
@@ -137,7 +145,9 @@ static GtkWidget *draw_desktop_checkbox_widget;
 
 /* `Launch Medusa' globals */
 static gboolean medusa_is_blocked;
+static NautilusCronStatus cron_status;
 static gboolean launch_medusa = TRUE;
+static GtkWidget *enable_medusa_checkbox_widget;
 
 static int current_user_level;
 
@@ -146,6 +156,7 @@ static gboolean set_http_proxy                   (const char 	*proxy_url);
 static gboolean attempt_http_proxy_autoconfigure (void);
 static gboolean check_network_connectivity	 (void);
 static void	convert_gmc_desktop_icons 	 (void);
+static void	update_finished_label		 (void);
 
 static void
 druid_cancel (GtkWidget *druid)
@@ -170,16 +181,16 @@ druid_set_first_time_file_flag (void)
 	user_directory = nautilus_get_user_directory ();
 	druid_flag_file_name = g_strdup_printf ("%s/%s",
 						user_directory,
-						"first-time-wizard-flag");
+						"first-time-flag");
 	g_free (user_directory);
 
 		
 	stream = fopen (druid_flag_file_name, "w");
 	if (stream) {
 		const char *blurb =
-			_("Existence of this file indicates that the Nautilus configuration wizard\n"
+			_("Existence of this file indicates that the Nautilus configuration druid\n"
 				"has been presented.\n\n"
-				"You can manually erase this file to present the wizard again.\n\n");
+				"You can manually erase this file to present the druid again.\n\n");
 			
 			fwrite (blurb, sizeof (char), strlen (blurb), stream);
 			fclose (stream);
@@ -205,6 +216,9 @@ druid_finished (GtkWidget *druid_page)
 	/* write out the first time file to indicate that we've successfully traversed the druid */
 	druid_set_first_time_file_flag ();
 
+	/* Do the user level config */
+	nautilus_preferences_set_user_level (current_user_level);
+	
 	/* Here we check to see if we can resolve hostnames in a timely
 	 * fashion. If we can't then we silently tell nautilus to start up
 	 * pointing to the home directory and not any of the HTTP addresses--
@@ -220,11 +234,7 @@ druid_finished (GtkWidget *druid_page)
 
 	signup_uris[0] = nautilus_preferences_get (NAUTILUS_PREFERENCES_HOME_URI);
 
-	if (Success == network_status) {
-		/* FIXME: even if the DNS resolution check didn't hang,
-		 * we still may not be able to load the services URL.
-		 * For example, we may not have a network connection!
-		 */
+	if (http_is_known_to_work) {
 		signup_uris[1] = EAZEL_SERVICES_URL;
 		signup_uris[2] = NULL;
 	} else {
@@ -232,9 +242,6 @@ druid_finished (GtkWidget *druid_page)
 	}
 
 
-	/* Do the user level config */
-	nautilus_preferences_set_user_level (current_user_level);
-	
 	/* Do the GMC to Nautilus Transition */
 	nautilus_preferences_set_boolean (NAUTILUS_PREFERENCES_SHOW_DESKTOP, draw_desktop);	
 	nautilus_preferences_set_boolean (NAUTILUS_PREFERENCES_ADD_TO_SESSION, add_to_session);	
@@ -244,6 +251,7 @@ druid_finished (GtkWidget *druid_page)
 
 	/* Do the Medusa config */
 	nautilus_preferences_set_boolean (NAUTILUS_PREFERENCES_USE_FAST_SEARCH, launch_medusa);
+	nautilus_medusa_enable_services (launch_medusa);
 	
 	/* Create default services icon on the desktop */
 	desktop_path = nautilus_get_desktop_directory ();
@@ -337,10 +345,18 @@ new_title_label (const char *text)
 
 	label = label_new_left_justified (text);
 
-	/* Note to localizers: this font is used for title text in
-	 * the first time druid */
-	font = nautilus_scalable_font_new (_("helvetica"), _("medium"), NULL, NULL);
-	nautilus_label_set_is_smooth (NAUTILUS_LABEL (label), TRUE);
+	font = nautilus_scalable_font_get_default_font ();
+
+	/* Force the label into smooth mode only if our locale is not
+	 * multibyte.  The problem is that for multibyte locales it is
+	 * likely that there will be no properly encoded font setup for
+	 * nautilus use at this time.  By making the label not smooth in 
+	 * multi bytes locales, we make the first time druid work in more
+	 * systems.
+	 */
+	nautilus_label_set_is_smooth (NAUTILUS_LABEL (label), 
+				      !nautilus_dumb_down_for_multi_byte_locale_hack ());
+
 	nautilus_label_set_smooth_font (NAUTILUS_LABEL (label), font);
 	nautilus_label_make_larger (NAUTILUS_LABEL (label), 10);
 	nautilus_label_set_smooth_drop_shadow_offset (NAUTILUS_LABEL (label), 2);
@@ -738,10 +754,27 @@ next_update_page_callback (GtkWidget *button, GnomeDruid *druid)
 	}
 
 	/* the user declined to update, so skip the feedback page and go directly to finish */
+	update_finished_label ();
 	gnome_druid_set_page (druid, GNOME_DRUID_PAGE (finish_page));
 	return TRUE;
 }
 
+/* handle the "next" signal for the update page based on the user's choice */
+static gboolean
+back_update_page_callback (GtkWidget *button, GnomeDruid *druid)
+{
+	/* If we didn't want medusa, or cron is active, don't go "back" to the cron page */
+	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (enable_medusa_checkbox_widget)) ||
+	    cron_status == NAUTILUS_CRON_STATUS_ON) {
+		gnome_druid_set_page (druid, GNOME_DRUID_PAGE (pages[LAUNCH_MEDUSA_PAGE]));
+	}
+	else {
+		gnome_druid_set_page (druid, GNOME_DRUID_PAGE (pages[START_CRON_INFORMATION_PAGE]));
+	}
+
+	return TRUE;
+}
+
 /* handle the "next" signal for the update feedback page to skip the error page */
 static gboolean
 next_update_feedback_page_callback (GtkWidget *button, GnomeDruid *druid)
@@ -888,10 +921,14 @@ set_up_gmc_transition_page (NautilusDruidPageEazel *page)
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbox), transfer_gmc_icons);	
 	gtk_signal_connect (GTK_OBJECT (checkbox), "toggled", GTK_SIGNAL_FUNC (transition_value_changed), &transfer_gmc_icons);
 
+#if 0
+	/* This option is currently disabled, per bugzilla.eazel.com 7557 */
+
 	checkbox = gtk_check_button_new_with_label (_("Launch Nautilus when GNOME starts up."));
 	gtk_box_pack_start (GTK_BOX (main_box), checkbox, FALSE, FALSE, 0);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbox), add_to_session);
 	gtk_signal_connect (GTK_OBJECT (checkbox), "toggled", GTK_SIGNAL_FUNC (transition_value_changed), &add_to_session);
+#endif
 	
 	gtk_widget_show_all (main_box);
 }
@@ -905,11 +942,74 @@ medusa_value_changed (GtkWidget *checkbox, gboolean *value)
 	return TRUE;
 }
 
-/* set up the "Launch Medusa" page */
+/* set up the "Launch Medusa" page, optionally with information about cron */
 static void
 set_up_medusa_page (NautilusDruidPageEazel *page)
 {
-	GtkWidget *checkbox, *label;
+	GtkWidget *label;
+	GtkWidget *container, *main_box, *hbox;
+
+	container = set_up_background (page, "rgb:ffff/ffff/ffff:h");
+	
+	/* Figure out whether cron is running */
+	cron_status = nautilus_medusa_check_cron_is_enabled ();
+
+	/* allocate a vbox to hold the description and the widgets */
+	main_box = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (main_box);
+	gtk_container_add (GTK_CONTAINER (container), main_box);
+	
+	/* allocate a descriptive label */
+	hbox = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (main_box), hbox, FALSE, FALSE, 0);
+
+	label = new_body_label (cron_status != NAUTILUS_CRON_STATUS_ON ?
+				_("The fast search feature indexes all the items on your hard disk.\n"
+				  "It speeds up searching and allows you to search by file content\n"
+				  "as well as filename, date, and other file properties.  The indexing\n"
+				  "takes time but is performed only when your computer is idle.\n"
+				  "Fast search requires that the cron daemon on your system be running.\n") :
+				_("The fast search feature indexes all the items on your hard disk.\n"
+				  "It speeds up searching and allows you to search by file content\n"
+				  "as well as filename, date, and other file properties. The indexing\n"
+				  "takes time but is performed only when your computer is idle.\n"));
+
+	gtk_widget_show (label);
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+
+	enable_medusa_checkbox_widget = gtk_check_button_new_with_label (cron_status == NAUTILUS_CRON_STATUS_ON ?
+						    _("Enable fast search") :
+						    _("Turn fast search on when cron is enabled"));
+	gtk_box_pack_start (GTK_BOX (main_box), enable_medusa_checkbox_widget, FALSE, FALSE, 0);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (enable_medusa_checkbox_widget), TRUE);	
+	gtk_signal_connect (GTK_OBJECT (enable_medusa_checkbox_widget), "toggled", GTK_SIGNAL_FUNC (medusa_value_changed), &launch_medusa);
+	
+	launch_medusa = TRUE;
+
+	gtk_widget_show_all (main_box);
+}
+
+/* handle the "next" signal for the medusa page, if cron is enabled. */
+static gboolean
+next_medusa_page_callback (GtkWidget *button, GnomeDruid *druid)
+{
+	/* If we didn't want medusa, or cron is active, continue as normal */
+	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (enable_medusa_checkbox_widget)) ||
+	    cron_status == NAUTILUS_CRON_STATUS_ON) {
+		gnome_druid_set_page (druid, GNOME_DRUID_PAGE (pages[OFFER_UPDATE_PAGE]));
+	}
+	else {
+		gnome_druid_set_page (druid, GNOME_DRUID_PAGE (pages[START_CRON_INFORMATION_PAGE]));
+	}
+
+	return TRUE;
+}
+
+/* set up the "Cron Information" page with information about cron */
+static void
+set_up_cron_information_page (NautilusDruidPageEazel *page)
+{
+	GtkWidget *label;
 	GtkWidget *container, *main_box, *hbox;
 
 	container = set_up_background (page, "rgb:ffff/ffff/ffff:h");
@@ -923,25 +1023,33 @@ set_up_medusa_page (NautilusDruidPageEazel *page)
 	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (main_box), hbox, FALSE, FALSE, 0);
 
-	label = new_body_label (_("The fast search feature indexes all the items on your hard disk.\n"
-				  "It speeds up searching and allows you to search by file content\n"
-				  "as well as filename, date, and other file properties. The indexing\n"
-				  "takes time but is performed only when your computer is idle.\n"));
+	/* Translators: Do not translate this text.  It has not been edited yet, and will be altered shortly. */
+	label = new_body_label (cron_status == NAUTILUS_CRON_STATUS_OFF ? 
+				_("Indexing is turned on, enabling the fast search feature. However, indexing\n"
+				  "currently can't be performed because the program crond, which does\n"
+				  "nightly tasks on your computer, is turned off. To make sure fast searches\n"
+				  "can be done, turn crond on.\n\n"
+				  "If you are running Linux, you can log in as root and type these commands \n"
+				  "to start cron:\n\n"
+				  "/sbin/chkconfig --level 345 crond on\n"
+				  "/etc/rc.d/init.d/cron start\n") :
+				_("Indexing is turned on, enabling the fast search feature. However, indexing\n"
+				  "may not be performed because the program crond, which does nightly tasks\n"
+				  "on your computer, may be turned off. To make sure fast searches can be\n"
+				  "done, check to make sure that crond is turned on.\n\n"
+				  "If you are running Linux, you can log in as root and type these commands\n"
+				  "to start cron:\n\n"
+				  "/sbin/chkconfig --level 345 crond on\n"
+				  "/etc/rc.d/init.d/cron start\n"));
 
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 
-	checkbox = gtk_check_button_new_with_label (_("Enable fast search"));
-	gtk_box_pack_start (GTK_BOX (main_box), checkbox, FALSE, FALSE, 0);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbox), TRUE);	
-	gtk_signal_connect (GTK_OBJECT (checkbox), "toggled", GTK_SIGNAL_FUNC (medusa_value_changed), &launch_medusa);
-	
-	launch_medusa = TRUE;
-
 	gtk_widget_show_all (main_box);
 }
 
 
+
 /* handle the "back" signal from the finish page to skip the feedback page */
 static gboolean
 finish_page_back_callback (GtkWidget *button, GnomeDruid *druid)
@@ -1007,6 +1115,24 @@ make_title_page_icon_box (void)
 	return hbox;
 }
 
+static void
+update_finished_label (void)
+{
+	if (http_is_known_to_work) {
+		gtk_label_set_text (GTK_LABEL (finished_label),
+				    _("Click Finish to launch Nautilus. You'll start with two\n"
+				      "Nautilus windows: one shows your home folder, and the\n"
+				      "other tells you about Eazel's services that make the life\n"
+				      "of a Linux user easier.\n\n"
+				      "We hope you enjoy Nautilus!"));
+	} else {
+		gtk_label_set_text (GTK_LABEL (finished_label),
+				    _("Click Finish to launch Nautilus. You'll start with a\n"
+				      "window showing your home folder.\n\n"
+				      "We hope you enjoy Nautilus!"));
+	}
+}
+
 GtkWidget *
 nautilus_first_time_druid_show (NautilusApplication *application, gboolean manage_desktop, const char *urls[])
 {	
@@ -1057,13 +1183,12 @@ nautilus_first_time_druid_show (NautilusApplication *application, gboolean manag
 	set_page_title (NAUTILUS_DRUID_PAGE_EAZEL (start_page),
 			_("Welcome to Nautilus"));
 	
-	label = new_body_label ( _("Nautilus is part of your GNOME desktop environment. It\n"
-				   "helps you manage your files and folders (directories),\n"
-				   "view documents, connect to the Internet, and do many\n"
-				   "other tasks.\n\n"
-				   "The next four screens ask about your level of Linux\n"
-				   "experience, and check your Internet connection. Click Next\n"
-				   "to continue."));
+	label = new_body_label ( _("Nautilus...\n"
+				   "    Is a great file manager for the GNOME desktop.\n"
+				   "    Provides a simple intuitive way to access Internet services.\n"
+				   "    Is an extensible framework for GNOME developers.\n"
+				   "    Provides a powerful delivery platform for service providers.\n\n"
+				   "Click Next to begin customizing your Nautilus environment."));
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (main_box), label, FALSE, FALSE, 0);
 	gtk_box_pack_end (GTK_BOX (main_box), make_title_page_icon_box (), TRUE, TRUE, 32);
@@ -1076,14 +1201,11 @@ nautilus_first_time_druid_show (NautilusApplication *application, gboolean manag
 	gtk_widget_show (main_box);
 	gtk_container_add (GTK_CONTAINER (container), main_box);
 	
-	label = new_body_label ( _("Click Finish to launch Nautilus. You'll start with two\n"
-				   "Nautilus windows: one shows your home folder, and the\n"
-				   "other tells you about Eazel's services that make the life\n"
-				   "of a Linux user easier.\n\n"
-				   "We hope you enjoy Nautilus!"));
+	label = new_body_label ("");
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (main_box), label, FALSE, FALSE, 0);
-		
+	finished_label = label;
+
 	/* set up the user level page */
 	set_page_title (NAUTILUS_DRUID_PAGE_EAZEL (pages[USER_LEVEL_PAGE]), _("Choose Your User Level"));
 	set_up_user_level_page (NAUTILUS_DRUID_PAGE_EAZEL (pages[USER_LEVEL_PAGE]));
@@ -1095,6 +1217,15 @@ nautilus_first_time_druid_show (NautilusApplication *application, gboolean manag
 	/* set up the `Launch Medusa' page */
 	set_page_title (NAUTILUS_DRUID_PAGE_EAZEL (pages[LAUNCH_MEDUSA_PAGE]), _("Fast Searches"));
 	set_up_medusa_page (NAUTILUS_DRUID_PAGE_EAZEL (pages[LAUNCH_MEDUSA_PAGE]));
+	gtk_signal_connect (GTK_OBJECT (pages[LAUNCH_MEDUSA_PAGE]), "next",
+			    GTK_SIGNAL_FUNC (next_medusa_page_callback),
+			    druid);
+
+
+	/* set up optional page to tell the user how to run cron */
+	set_page_title (NAUTILUS_DRUID_PAGE_EAZEL (pages[START_CRON_INFORMATION_PAGE]), _("The Cron Daemon"));
+	set_up_cron_information_page (NAUTILUS_DRUID_PAGE_EAZEL (pages[START_CRON_INFORMATION_PAGE]));
+	
 
 	/* set up the update page */
 	set_page_title (NAUTILUS_DRUID_PAGE_EAZEL (pages[OFFER_UPDATE_PAGE]), _("Checking Your Internet Connection"));
@@ -1103,6 +1234,9 @@ nautilus_first_time_druid_show (NautilusApplication *application, gboolean manag
 	gtk_signal_connect (GTK_OBJECT (pages[OFFER_UPDATE_PAGE]), "next",
 			    GTK_SIGNAL_FUNC (next_update_page_callback),
 			    druid);
+	gtk_signal_connect (GTK_OBJECT (pages[OFFER_UPDATE_PAGE]), "back",
+			    GTK_SIGNAL_FUNC (back_update_page_callback),
+			    druid);
 
 	/* set up the update feedback page */
 	set_page_title (NAUTILUS_DRUID_PAGE_EAZEL (pages[UPDATE_FEEDBACK_PAGE]), _("Updating Nautilus..."));
@@ -1211,6 +1345,8 @@ download_callback (GnomeVFSResult result,
 		g_free (untar_command);
 		g_free (remove_command);
 
+		http_is_known_to_work = TRUE;
+
 		/* now that we're done, reenable the buttons */
 		gnome_druid_set_buttons_sensitive (druid, TRUE, TRUE, TRUE);
 	} else if (result == GNOME_VFS_ERROR_NOT_FOUND) {
@@ -1222,6 +1358,8 @@ download_callback (GnomeVFSResult result,
 
 		/* now that we're done, reenable the buttons */
 		gnome_druid_set_buttons_sensitive (druid, TRUE, TRUE, TRUE);
+
+		http_is_known_to_work = TRUE;
 	} else {
 		/* there was an error; see if we can't find some HTTP proxy config info */
 		/* note that attempt_http_proxy_autoconfigure returns FALSE if it's already been tried */ 
@@ -1232,6 +1370,11 @@ download_callback (GnomeVFSResult result,
 			gnome_druid_set_page (druid, GNOME_DRUID_PAGE (pages[PROXY_CONFIGURATION_PAGE]));
 		}
 	}
+
+	/* Make sure the state of the final body text reflects
+	 * the number of pages we'll be showing initially.
+	 */
+	update_finished_label ();
 }
 
 /* initiate downloading of the welcome package from the service */
