@@ -36,6 +36,7 @@
 #include <gtk/gtklabel.h>
 #include <gtk/gtkpixmap.h>
 #include <gtk/gtksignal.h>
+#include <libgnome/gnome-i18n.h>
 #include <libgnomevfs/gnome-vfs-types.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libnautilus-extensions/nautilus-background.h>
@@ -68,7 +69,8 @@ static void                nautilus_sidebar_title_theme_changed    (gpointer
 static void                update_icon                             (NautilusSidebarTitle      *sidebar_title);
 static GtkWidget *         sidebar_title_create_title_label        (void);
 static GtkWidget *         sidebar_title_create_more_info_label    (void);
-static void                smooth_graphics_mode_changed_callback   (gpointer                   callback_data);
+static void                update_all_cover   			   (gpointer                   callback_data);
+static void		   update_all 				   (NautilusSidebarTitle      *sidebar_title);
 static NautilusBackground *nautilus_sidebar_title_background       (NautilusSidebarTitle      *sidebar_title);
 
 struct NautilusSidebarTitleDetails {
@@ -82,6 +84,7 @@ struct NautilusSidebarTitleDetails {
 	GtkWidget		*notes;
 
 	int			shadow_offset;
+	gboolean		determined_icon;
 };
 
 NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusSidebarTitle, nautilus_sidebar_title, gtk_vbox_get_type ())
@@ -121,6 +124,27 @@ realize_callback (NautilusSidebarTitle *sidebar_title)
 					GTK_OBJECT (sidebar_title));
 }
 
+static void
+smooth_font_changed_callback (gpointer callback_data)
+{
+	NautilusScalableFont *new_font;
+	NautilusScalableFont *new_bold_font;
+	NautilusSidebarTitle *sidebar_title;
+
+	g_return_if_fail (NAUTILUS_IS_SIDEBAR_TITLE (callback_data));
+
+	sidebar_title = NAUTILUS_SIDEBAR_TITLE (callback_data);
+	
+	new_font = nautilus_global_preferences_get_smooth_font ();
+	new_bold_font = nautilus_global_preferences_get_smooth_bold_font ();
+
+	nautilus_label_set_smooth_font (NAUTILUS_LABEL (sidebar_title->details->title_label), new_bold_font);
+	nautilus_label_set_smooth_font (NAUTILUS_LABEL (sidebar_title->details->more_info_label), new_font);
+
+	gtk_object_unref (GTK_OBJECT (new_font));
+	gtk_object_unref (GTK_OBJECT (new_bold_font));
+}
+
 static void
 nautilus_sidebar_title_initialize (NautilusSidebarTitle *sidebar_title)
 { 
@@ -152,15 +176,26 @@ nautilus_sidebar_title_initialize (NautilusSidebarTitle *sidebar_title)
 	gtk_widget_show (sidebar_title->details->emblem_box);
 	gtk_box_pack_start (GTK_BOX (sidebar_title), sidebar_title->details->emblem_box, 0, 0, 0);
 
+	/* FIXME: This should use NautilusLabel like the other displayed text.
+	 * But I don't think this feature is ever used? Someone should consult
+	 * with Andy about this. (This is not the same as the info in the Notes
+	 * sidebar panel.)
+	 */
 	sidebar_title->details->notes = GTK_WIDGET (gtk_label_new (NULL));
 	gtk_label_set_line_wrap (GTK_LABEL (sidebar_title->details->notes), TRUE);
 	gtk_widget_show (sidebar_title->details->notes);
 	gtk_box_pack_start (GTK_BOX (sidebar_title), sidebar_title->details->notes, 0, 0, 0);
 
 	/* Keep track of changes in graphics trade offs */
-	smooth_graphics_mode_changed_callback (sidebar_title);
+	update_all (sidebar_title);
 	nautilus_preferences_add_callback (NAUTILUS_PREFERENCES_SMOOTH_GRAPHICS_MODE, 
-					   smooth_graphics_mode_changed_callback, 
+					   update_all_cover, 
+					   sidebar_title);
+	nautilus_preferences_add_callback (NAUTILUS_PREFERENCES_DIRECTORY_VIEW_FONT_FAMILY,
+					   update_all_cover,
+					   sidebar_title);
+	nautilus_preferences_add_callback (NAUTILUS_PREFERENCES_DIRECTORY_VIEW_SMOOTH_FONT,
+					   smooth_font_changed_callback,
 					   sidebar_title);
 
 	/* set up the label colors according to the theme, and get notified of changes */
@@ -198,8 +233,18 @@ nautilus_sidebar_title_destroy (GtkObject *object)
 	g_free (sidebar_title->details->title_text);
 	g_free (sidebar_title->details);
 
-	nautilus_preferences_remove_callback (NAUTILUS_PREFERENCES_THEME, nautilus_sidebar_title_theme_changed, sidebar_title);
-	nautilus_preferences_remove_callback (NAUTILUS_PREFERENCES_SMOOTH_GRAPHICS_MODE, smooth_graphics_mode_changed_callback, sidebar_title);
+	nautilus_preferences_remove_callback (NAUTILUS_PREFERENCES_THEME, 
+					      nautilus_sidebar_title_theme_changed, 
+					      sidebar_title);
+	nautilus_preferences_remove_callback (NAUTILUS_PREFERENCES_SMOOTH_GRAPHICS_MODE, 
+					      update_all_cover, 
+					      sidebar_title);
+	nautilus_preferences_remove_callback (NAUTILUS_PREFERENCES_DIRECTORY_VIEW_FONT_FAMILY, 
+					      update_all_cover, 
+					      sidebar_title);
+	nautilus_preferences_remove_callback (NAUTILUS_PREFERENCES_DIRECTORY_VIEW_SMOOTH_FONT,
+					      smooth_font_changed_callback,
+					      sidebar_title);
   	
 	NAUTILUS_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
@@ -359,11 +404,13 @@ update_icon (NautilusSidebarTitle *sidebar_title)
 {
 	GdkPixbuf *pixbuf;
 	char *uri, *icon_path;
+	gboolean leave_pixbuf_unchanged;
 	
 	/* FIXME bugzilla.eazel.com 5043: currently, components can't specify their own sidebar icon.  This
 	  needs to be added to the framework, but for now we special-case some
 	  important ones here */
-	
+
+	leave_pixbuf_unchanged = FALSE;
 	uri = NULL;
 	icon_path = NULL;
 	if (sidebar_title->details->file) {
@@ -379,7 +426,8 @@ update_icon (NautilusSidebarTitle *sidebar_title)
 	} else if (nautilus_istr_has_prefix (uri, "hardware:")) {
 		icon_path = nautilus_theme_get_image_path ("computer.svg");
 	}
-	
+
+	pixbuf = NULL;
 	if (icon_path != NULL) {
 		pixbuf = nautilus_icon_factory_get_pixbuf_from_name (icon_path, NULL, NAUTILUS_ICON_SIZE_LARGE, TRUE);
 	} else if (nautilus_icon_factory_is_icon_ready_for_file (sidebar_title->details->file)) {
@@ -387,14 +435,26 @@ update_icon (NautilusSidebarTitle *sidebar_title)
 								    "accept",
 								    NAUTILUS_ICON_SIZE_LARGE,
 								    TRUE);
-	} else {
-		pixbuf = NULL;
+	} else if (sidebar_title->details->determined_icon) {
+		/* We used to know the icon for this file, but now the file says it isn't
+		 * ready. This means that some file info has been invalidated, which
+		 * doesn't necessarily mean that the previously-determined icon is
+		 * wrong (in fact, in practice it usually doesn't mean that). Keep showing
+		 * the one we last determined for now.
+		 */
+		 leave_pixbuf_unchanged = TRUE;
 	}
-
+	
 	g_free (uri);	
 	g_free (icon_path);
-	
-	nautilus_image_set_pixbuf (NAUTILUS_IMAGE (sidebar_title->details->icon), pixbuf);
+
+	if (pixbuf != NULL) {
+		sidebar_title->details->determined_icon = TRUE;
+	}
+
+	if (!leave_pixbuf_unchanged) {
+		nautilus_image_set_pixbuf (NAUTILUS_IMAGE (sidebar_title->details->icon), pixbuf);
+	}
 }
 
 static void
@@ -437,7 +497,7 @@ update_font (NautilusSidebarTitle *sidebar_title)
 	/* FIXME bugzilla.eazel.com 1103: Hard coded font family. */
 	
 	/* Update the regular font */
-	template_font = nautilus_font_factory_get_font_by_family ("helvetica", maximum_acceptable_font_size);
+	template_font = nautilus_font_factory_get_font_by_family (_("helvetica"), maximum_acceptable_font_size);
 	g_assert (template_font != NULL);
 
 	bold_template_font = nautilus_gdk_font_get_bold (template_font);
@@ -652,6 +712,7 @@ nautilus_sidebar_title_set_file (NautilusSidebarTitle *sidebar_title,
 	if (file != sidebar_title->details->file) {
 		release_file (sidebar_title);
 		sidebar_title->details->file = file;
+		sidebar_title->details->determined_icon = FALSE;
 		nautilus_file_ref (sidebar_title->details->file);
 	
 		/* attach file */
@@ -734,7 +795,7 @@ sidebar_title_create_more_info_label (void)
 }
 
 static void
-smooth_graphics_mode_changed_callback (gpointer callback_data)
+update_all_cover (gpointer callback_data)
 {
 	g_return_if_fail (NAUTILUS_IS_SIDEBAR_TITLE (callback_data));
 	
