@@ -29,17 +29,16 @@
 
 #include "nautilus-window.h"
 
-void nautilus_window_manage_views_destroy        (NautilusWindow             *window);
-void nautilus_window_open_location               (NautilusWindow             *window,
-                                                  const char                 *location);
-void nautilus_window_stop_loading                (NautilusWindow             *window);
-void nautilus_window_set_content_view            (NautilusWindow             *window,
-                                                  NautilusViewIdentifier     *id);
-void nautilus_window_set_sidebar_panels          (NautilusWindow             *window,
-                                                  GList                      *view_identifier_list);
-void nautilus_window_begin_location_change       (NautilusWindow             *window,
-                                                  const char                 *location,
-                                                  NautilusLocationChangeType  type,
-                                                  guint                       distance);
+void nautilus_window_manage_views_destroy (NautilusWindow         *window);
+void nautilus_window_open_location        (NautilusWindow         *window,
+                                           const char             *location);
+void nautilus_window_stop_loading         (NautilusWindow         *window);
+void nautilus_window_set_content_view     (NautilusWindow         *window,
+                                           NautilusViewIdentifier *id);
+void nautilus_window_set_sidebar_panels   (NautilusWindow         *window,
+                                           GList                  *view_identifier_list);
+void nautilus_window_back_or_forward      (NautilusWindow         *window,
+                                           gboolean                back,
+                                           guint                   distance);
 
 #endif /* NAUTILUS_WINDOW_MANAGE_VIEWS_H */
